import string
from wxPython.wx import *
from wxPython.grid import *
from wxPython.lib.evtmgr import eventManager

FORBIDDEN_KEYS = (WXK_F1, WXK_F2, WXK_F9, WXK_F12, WXK_CONTROL, WXK_ALT)

class AttribGrid(wxGrid):
	
	def __init__(self, master, *args, **kwargs):
		apply(wxGrid.__init__,(self,) + args, kwargs)
		self.master = master
		self.edit = 1
		
		table = AttribDataTable()
		self.SetTable(table, 1)

		self.SetRowLabelSize(0)
		self.AutoSizeColumns()
		self.SetMargins(50,50)
		self.Fit()
		attr0 = wxGridCellAttr()
		attr0.SetReadOnly(1)
		self.SetColAttr(0, attr0)
		attr1 = wxGridCellAttr()
		attr1.SetEditor(AttribGridEditor(self.master))
		self.SetColAttr(1, attr1)

		self.attribute = self.value = ''

		eventManager.Register(self.onLeftDClick,	 EVT_GRID_CELL_LEFT_DCLICK, self)

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def onLeftDClick(self, evt):
		if self.CanEnableCellControl():
			self.EnableCellEditControl()
		
	def ClearGrid(self):
		self.GetTable().data = [['','']]
		self.DeleteRows(0, self.GetNumberRows()-1)
		
class AttribGridEditor(wxPyGridCellEditor):

	def __init__(self, master, *args, **kw):
		apply(wxPyGridCellEditor.__init__,(self,) + args, kw)
		self.master = master

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def Create(self, parent, id, evtHandler):
		"""
		Called to create the control, which must derive from wxControl.
		*Must Override*
		"""
		self._tc = wxTextCtrl(parent, id, '')
		self._tc.SetInsertionPoint(0)
		self.SetControl(self._tc)
		if evtHandler:
			self._tc.PushEventHandler(evtHandler)

	def IsAcceptedKey(self, evt):
		return (not (evt.ControlDown() or evt.AltDown()) and
				evt.GetKeyCode() not in FORBIDDEN_KEYS)

	def BeginEdit(self, row, col, grid):
		"""
		Fetch the value from the table and prepare the edit control
		to begin editing.  Set the focus to the edit control.
		*Must Override*
		"""
		self.startValue	 = grid.GetTable().GetValue(row, col)
		self.startAttrib = grid.GetTable().GetValue(row, col-1)
		self._tc.SetValue(self.startValue)
		self._tc.SetInsertionPointEnd()
		self._tc.SetFocus()

		self._tc.SetSelection(0, self._tc.GetLastPosition())
			
	def EndEdit(self, r, c, grid):
		changed = 0
		val = string.strip(self._tc.GetValue())
		if val != self.startValue and val != '':
			changed = 1
			grid.GetTable().SetValue(r, c, val) # update the table

			if self.master.tree.changing:
				node = self.master.tree.last
			else:
				node = self.master.master.tree.GetSelection()
				
			data = self.master.tree.GetPyData(node)
			data['attributes'][self.startAttrib] = val
			self.master.tree.SetPyData(node, data)
			if self.startAttrib == u'name':
				# Update the tree-nodes
				add = ' ' + val
				self.master.tree.SetItemText(node, data['name']+add)
				self.master.tree.SortChildren(self.master.tree.GetRootItem())
			if self.startAttrib == u'outfile':
				self.master.outfile = val
			if data['name'] == u'shader' and self.startAttrib == u'name':
				self.master.updateShaders(self.master.tree.GetRootItem(), 0, self.startValue, val)

			# Update shaderpanel if editing shaders
			if self.master.master.curShader:
				if self.master.master.tree.GetPyData(node).get('name', '') == u'shader':
					shader = node
					self.master.master.shaderp.setValues(node, self.master.master.curObj)
				else:
					shader	= self.master.master.traverseTo(self.master.master.tree.GetSelection(), u'shader')
					if shader:
						self.master.master.shaderp.setValues(shader, self.master.master.curObj)
						
			# Update lightpanel if editing lights
			if self.master.master.curLight:
				if self.master.master.tree.GetPyData(node).get('name', '') == u'light':
					if node == self.master.master.curLight:
						self.master.master.lightp.setValues(self.master.master.curLight)
				else:
					light	= self.master.master.traverseTo(self.master.master.tree.GetSelection(), u'light')
					if light == self.master.master.curLight:
						self.master.master.lightp.setValues(light)

		self.startValue	 = ''
		self.startAttrib = ''
		self._tc.SetValue('')
		self.master.master.setChanged(1)
		return changed

	def Reset(self):
		"""
		Reset the value in the control back to its starting value.
		*Must Override*
		"""
		self._tc.SetValue(self.startValue)
		self._tc.SetInsertionPointEnd()

	def Clone(self):
		"""
		Create a new object which is the copy of this one
		*Must Override*
		"""
		return AttribGridEditor(self.master.tree)
		
class AttribGridColorEditor(wxPyGridCellEditor):

	def __init__(self, master, color, *args, **kw):
		apply(wxPyGridCellEditor.__init__,(self,) + args, kw)
		self.master = master
		self.reset = 0
		self.colordata = wxColourData()
		self.colordata.SetColour(wxColour(int(color[0]*255), int(color[1]*255), int(color[2]*255)))
		self.colordata.SetChooseFull(1)
		
	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def Create(self, parent, id, evtHandler):
		"""
		Called to create the control, which must derive from wxControl.
		*Must Override*
		"""
		self._tc = wxTextCtrl(parent, id, '')
		self._tc.SetInsertionPoint(0)
		self.SetControl(self._tc)
		if evtHandler:
			self._tc.PushEventHandler(evtHandler)

	def IsAcceptedKey(self, evt):
		return (not (evt.ControlDown() or evt.AltDown()) and
				evt.GetKeyCode() not in FORBIDDEN_KEYS)

	def BeginEdit(self, row, col, grid):
		"""
		Fetch the value from the table and prepare the edit control
		to begin editing.  Set the focus to the edit control.
		*Must Override*
		"""
		self.startValue = grid.GetTable().GetValue(row, col)
		self._tc.SetValue(grid.GetTable().GetValue(row, col))
		self._tc.SetInsertionPointEnd()
		self._tc.SetFocus()
		self.outdlg = None
		dlg = wxColourDialog(self.master, self.colordata)
		if dlg.ShowModal() == wxID_OK:
			self.outdlg = dlg.GetColourData()
			self.colordata.SetColour(wxColour(self.outdlg.GetColour().Red(),
									 self.outdlg.GetColour().Green(),
									 self.outdlg.GetColour().Blue()))
			color = self.colordata.GetColour()
			self._tc.SetValue('(%s, %s, %s)' % (self.colordata.GetColour().Red()/255.0,
												self.colordata.GetColour().Green()/255.0,
												self.colordata.GetColour().Blue()/255.0))
		dlg.Destroy()
		wxCallAfter(grid.DisableCellEditControl) # Thanks to Robin Dunn for this :)))
		
	def EndEdit(self, r, c, grid):
		"""
		Complete the editing of the current cell. Returns true if the value
		has changed.  If necessary, the control may be destroyed.
		*Must Override*
		"""
		changed = 0
		if self.outdlg and not self.reset:
			grid.GetTable().SetValue(r, c, self._tc.GetValue()) # update the table
			grid.GetCellRenderer(r, c).color = [self.colordata.GetColour().Red(),
												self.colordata.GetColour().Green(),
												self.colordata.GetColour().Blue()]
			if self.master.tree.changing:
				node = self.master.tree.last
			else:
				node = self.master.tree.GetSelection()
			treedata = self.master.tree.GetPyData(node)
			treedata['attributes']['r'] = "%s" % (self.colordata.GetColour().Red()/255.0)
			treedata['attributes']['g'] = "%s" % (self.colordata.GetColour().Green()/255.0)
			treedata['attributes']['b'] = "%s" % (self.colordata.GetColour().Blue()/255.0)
			self.master.tree.SetPyData(node, treedata)
			changed = 1
			self.master.master.setChanged(1)

			# Update shaderpanel if editing shaders
			if self.master.master.curShader:
				if self.master.master.tree.GetPyData(node).get('name', '') == u'shader':
					shader = node
					self.master.master.shaderp.setValues(node, self.master.master.curObj)
				else:
					shader	= self.master.master.traverseTo(self.master.master.tree.GetSelection(), u'shader')
					if shader:
						self.master.master.shaderp.setValues(shader, self.master.master.curObj)
						
			# Update lightpanel if editing lights
			if self.master.master.curLight:
				if self.master.master.tree.GetPyData(node).get('name', '') == u'light':
					if node == self.master.master.curLight:
						self.master.master.lightp.setValues(self.master.master.curLight)
				else:
					light	= self.master.master.traverseTo(self.master.master.tree.GetSelection(), u'light')
					if light == self.master.master.curLight:
						self.master.master.lightp.setValues(light)

		self._tc.SetValue('')
		return changed

	def Reset(self):
		"""
		Reset the value in the control back to its starting value.
		*Must Override*
		"""
		self.reset = 1
		self._tc.SetValue(self.startValue)
		self._tc.SetInsertionPointEnd()

	def Clone(self):
		"""
		Create a new object which is the copy of this one
		*Must Override*
		"""
		return AttribGridColorEditor(self.master.tree)
		
class AttribDataTable(wxPyGridTableBase):
	
	def __init__(self):
		wxPyGridTableBase.__init__(self)
		self.colLabels = ['Attribut', 'Value']
		self.dataTypes = [wxGRID_VALUE_STRING, wxGRID_VALUE_STRING]
		self.data	   = [['', '']]*30
		
	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	# required methods for the wxPyGridTableBase interface
		
	def GetNumberRows(self):
		return len(self.data)
	
	def GetNumberCols(self):
		return len(self.data[0])
	
	def IsEmptyCell(self, row, col):
		try:
			return not self.data[row][col]
		except IndexError:
			return 1
		
	def GetValue(self, row, col):
		try:
			return self.data[row][col]
		except IndexError:
			return ''
		
	def SetValue(self, row, col, value):
		try:
			self.data[row][col] = value
		except IndexError:
			# add a new row
			self.data.append([''] * self.GetNumberCols())
			self.SetValue(row, col, value)
			
			if row >= self.GetView().GetNumberRows():
				# tell the grid we've added a row
				msg = wxGridTableMessage(self, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1)
				self.GetView().ProcessTableMessage(msg)
			
	def DeleteRows(self, pos, num_rows):
		pass

	def Clear(self):
		pass
	
	# Some optional methods
	
	# Called when the grid needs to display labels
	def GetColLabelValue(self, col):
		return self.colLabels[col]
	
	# Called to determine the kind of editor/renderer to use by
	# default, doesn't necessarily have to be the same type used
	# nativly by the editor/renderer if they know how to convert.
	def GetTypeName(self, row, col):
		return self.dataTypes[col]

	# Called to determine how the data can be fetched and stored by the
	# editor and renderer.	This allows you to enforce some type-safety
	# in the grid.
	def CanGetValueAs(self, row, col, typeName):
		colType = self.dataTypes[col].split(':')[0]
		if typeName == colType:
			return 1
		else:
			return 0
		
	def CanSetValueAs(self, row, col, typeName):
		return self.CanGetValueAs(row, col, typeName)
		
class AttribGridColorRenderer(wxPyGridCellRenderer):

	def __init__(self, color):
		wxPyGridCellRenderer.__init__(self)
		self.color = [int(x*255) for x in color]
		
	def __repr__(self):
		return '<%s, address: %s, color: >' % (self.__class__.__name__, id(self), self.color)
		
	def Draw(self, grid, attr, dc, rect, row, col, isSelected):
		dc.SetBackgroundMode(wxSOLID)
		dc.SetBrush(wxBrush(wxColour(self.color[0], self.color[1], self.color[2]), wxSOLID))
		dc.SetPen(wxTRANSPARENT_PEN)
		dc.DrawRectangle(rect.x, rect.y, rect.width, rect.height)

		dc.SetBackgroundMode(wxTRANSPARENT)
		dc.SetFont(attr.GetFont())

		text = grid.GetCellValue(row, col)
##		  colors = [wxRED, wxWHITE, wxCYAN]
##		  x = rect.x + 1
##		  y = rect.y + 1
##		  for ch in text:
##			  dc.SetTextForeground(random.choice(colors))
##			  dc.DrawText(ch, x, y)
##			  w, h = dc.GetTextExtent(ch)
##			  x = x + w
##			  if x > rect.right - 5:
##				  break


	def GetBestSize(self, grid, attr, dc, row, col):
		text = grid.GetCellValue(row, col)
		dc.SetFont(attr.GetFont())
		w, h = dc.GetTextExtent(text)
		return wxSize(w, h)

	def Clone(self):
		return AttribGridColorRenderer()
