#! /usr/bin/env python
#
# Bagisas
#
# A GUI for yafray (www.yafray.org)
#
# Copyright (C) 2003 Wolfram Kraus (schlops@web.de)
#
# Very special thanks to:
#
#  - ray_theway(rayae1@comcast.net)
#	 for great inspiration and some code
#
#  - Stefano <S68> Selleri for the YaBleThumb
#	 script www.selleri.org/Blender
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA	 02111-1307, USA.

import profile

import os, os.path, popen2, string, sys, time, threading, traceback
import ConfigParser
import Render
from types import *
from TreePanel import *
from ShaderPanel import *
from LightPanel import *
from Dialogs import *
from xml.dom.ext.reader.Sax2 import FromXmlFile
from wxPython.wx import *
from wxPython.lib.evtmgr import eventManager

VERSION = '0.1.0'

# IDs:
ID_PREVIEW	  = wxNewId()
#  Menu
#  - File
ID_FNEW	   = wxNewId()
ID_FOPEN   = wxNewId()
ID_FSAVE   = wxNewId()
ID_FSAAS   = wxNewId()
ID_FPREF   = wxNewId()
ID_FPRES   = wxNewId()
ID_FEXIT   = wxNewId()

#  - Edit
ID_ET2S	   = wxNewId()
ID_ET2L	   = wxNewId()

#  - Tree
ID_TCOLA   = wxNewId()
ID_TCOLC   = wxNewId()
ID_TEXPA   = wxNewId()
ID_TEXPC   = wxNewId()

#  - Render
ID_RNDR	   = wxNewId()
ID_RNPR	   = wxNewId()
ID_RNTH	   = wxNewId()
ID_RSTP	   = wxNewId()

#  - Shader
ID_SHSA    = wxNewId()

#  - Help
ID_HABOUT  = wxNewId()
ID_HTESTY  = wxNewId()

#  Toolbar

MENU_STYLE = wxMENU_TEAROFF

class Bagisas(wxFrame):
	''' The main class, responsible for the window layout and eventhandling'''
	
	def __init__(self, parent, id, title, file = None):
		''' Constructor '''
		self.changed			= 0
		self.curShader			= None
		self.curLight			= None
		self.curObj				= None
		self.remove				= []
		self.outfile = 'salida.tga'
		self.config = { 'render.yafex'       : 'yafray',
						'render.numcpu'      : 1,
						'render.netop'       : 0,
						'render.scedir'      : '',
						'preview.defshader'  : 1,
						'preview.shader'     : '(200, 200, 200)',
						'preview.camlight'   : 1,
						'preview.lightpower' : 100,
						'preview.antialias'  : 1,
						'preview.size'       : 0,
						'preview.sizerx'     : 320,
						'preview.sizery'     : 240
						}

		self.loadConfig(['bagisas.ini', os.path.expanduser('~/.bagisasrc')])
		if file:
			self.dirname		= os.path.abspath(os.path.dirname(file))
			self.file				= os.path.basename(file)
		else:
			self.dirname        = self.config['render.scedir']
			self.file           = ''
			
		wxFrame.__init__(self,
						 parent,
						 id,
						 title,
						 style=wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE)
		
		self.SetDimensions(-1, -1, 770, 680)
		self.SetSizeHints(320, 200)
		
		sb = self.CreateStatusBar() 
		
		# Setting up the menus
		filemenu= wxMenu()
		filemenu.Append(ID_FNEW	  , '&New\tCtrl-X',		 ' Create A New Yafray-File')
		filemenu.Append(ID_FOPEN  , '&Open\tF1',		 ' Open An Existing Yafray-File')
		filemenu.AppendSeparator()
		filemenu.Append(ID_FSAVE  , '&Save\tCtrl-S',	 ' Save The Current Yafray-File')
		filemenu.Append(ID_FSAAS  , 'Save &As...\tF2',	 ' Save The Current Yafray-File As ...')
		filemenu.AppendSeparator()
		filemenu.Append(ID_FPREF  , '&Preferences',		 ' Set The Render Preferences')
		filemenu.Append(ID_FPRES  , 'Save Pre&ferences', ' Save The Render Preferences For Future Sessions')
		filemenu.AppendSeparator()
		filemenu.Append(ID_FEXIT  , 'E&xit\tCtrl-Q',	 ' Terminate The Program')
		
		editmenu= wxMenu()
		editmenu.Append(ID_ET2S	 , 'Copy to &Shaderpanel',  ' Copy currently selected Shader to Shaderpanel')
		editmenu.Append(ID_ET2L	 , 'Copy to &Lightpanel',   ' Copy currently selected Light to Lightpanel')

		treemenu= wxMenu()
		treemenu.Append(ID_TCOLA  , '&Collapse Complete Tree',	  ' Collapse every treenode')
		treemenu.Append(ID_TCOLC  , 'C&ollapse Current Node',	  ' Collapse the current selected treenode')
		treemenu.AppendSeparator()
		treemenu.Append(ID_TEXPA  , '&Expand Complete Tree',	  ' Expand every treenode')
		treemenu.Append(ID_TEXPC  , 'E&xpand Current Node',		  ' Expand the current selected treenode')

		rndrmenu= wxMenu()
		rndrmenu.Append(ID_RNDR	  , 'Render\tF12',		   ' Render Scene')
		rndrmenu.Append(ID_RNPR	  , 'Preview\tF9',		   ' Preview Scene')
#		rndrmenu.Append(ID_RSTP	  , 'Stop Render',		   ' Stop Renderthread')
		
		shdrmenu= wxMenu()
		shdrmenu.Append(ID_SHSA	  , 'Save Shader-Code',	' Save the shadercode to textfile')

		helpmenu= wxMenu()
		helpmenu.Append(ID_HABOUT , '&About',	   ' Informations About This Nice Little Program')
		helpmenu.Append(ID_HTESTY , '&Test Yafray',' Test whether your yafray path is set up correct.')
		
		# Create the menubar
		menuBar = wxMenuBar()
		menuBar.Append(filemenu,'&File') 
		menuBar.Append(editmenu,'&Edit') 
		menuBar.Append(treemenu,'&Tree') 
		menuBar.Append(rndrmenu,'&Render') 
		menuBar.Append(shdrmenu,'&Shader') 
		menuBar.Append(helpmenu,'&Help') 
		self.SetMenuBar(menuBar)  
		
		self.notebook = wxNotebook(self, -1)

		self.CentreOnScreen()
		self.Show(1)
		self.Raise()

		self.treep = TreePanel(self, self.notebook, -1)
		self.tree = self.treep.tree
		self.shaderp = ShaderPanel(self, self.notebook, -1)
		self.lightp = LightPanel(self, self.notebook, -1)
		
		self.notebook.AddPage(self.treep, 'Tree')
		self.notebook.AddPage(self.shaderp, 'Shader')
		self.notebook.AddPage(self.lightp, 'Light')
		self.notebook.SetSelection(0)

		# Set the Renderer
		self.render = None
		
		# Do the eventhandling
		self.setEventHandling()
		
		# Finally...
		
	def setEventHandling(self):
		''' Connects GUI objects with events'''
		# Eventhandling
		eventManager.Register(self.onClose,	   EVT_CLOSE, self)
		# - Menus
		eventManager.Register(self.onNew,		   EVT_MENU, win=self, id=ID_FNEW)
		eventManager.Register(self.onOpen,		   EVT_MENU, win=self, id=ID_FOPEN)
		eventManager.Register(self.onSave,		   EVT_MENU, win=self, id=ID_FSAVE)
		eventManager.Register(self.onSaveAs,	   EVT_MENU, win=self, id=ID_FSAAS)
		eventManager.Register(self.onPrefs,		   EVT_MENU, win=self, id=ID_FPREF)
		eventManager.Register(self.onSavePrefs,	   EVT_MENU, win=self, id=ID_FPRES)
		eventManager.Register(self.onExit,		   EVT_MENU, win=self, id=ID_FEXIT)
		eventManager.Register(self.onAbout,		   EVT_MENU, win=self, id=ID_HABOUT)
		eventManager.Register(self.onTestYaf,	   EVT_MENU, win=self, id=ID_HTESTY)
		eventManager.Register(self.onCopyToShader, EVT_MENU, win=self, id=ID_ET2S)
		eventManager.Register(self.onCopyToLight,  EVT_MENU, win=self, id=ID_ET2L)
		eventManager.Register(self.onRender,	   EVT_MENU, win=self, id=ID_RNDR)
		eventManager.Register(self.onPreview,	   EVT_MENU, win=self, id=ID_RNPR)
#		eventManager.Register(self.onThread,	   EVT_MENU, win=self, id=ID_RNTH)
#		eventManager.Register(self.onStopThread,   EVT_MENU, win=self, id=ID_RSTP)
		eventManager.Register(self.onSaveShader,   EVT_MENU, win=self, id=ID_SHSA)
		eventManager.Register(self.tree.onCollapse,EVT_MENU, win=self, id=ID_TCOLA)
		eventManager.Register(self.tree.onCollaC,  EVT_MENU, win=self, id=ID_TCOLC)
		eventManager.Register(self.tree.onExpand,  EVT_MENU, win=self, id=ID_TEXPA)
		eventManager.Register(self.tree.onExpandC, EVT_MENU, win=self, id=ID_TEXPC)
		
	def loadConfig(self, file):
		''' Opens An Existing Configfile If Available And Sets The Values'''
		cp = ConfigParser.ConfigParser(self.config)
		cp.read(file)
		for sec in cp.sections():
			name=string.lower(sec)
			for opt in cp.options(sec):
				self.config[name+'.'+string.lower(opt)]=string.strip(cp.get(sec, opt))
		
	def onNew(self, evt):
		if self.changed:
			dlg = wxMessageDialog(self,
								  'There are unsaved changes. Do you really want to create a new file?',
								  'Create new file?',
								  wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION) 
			if dlg.ShowModal() == wxID_YES: 
				self.createNew()
		else:
			self.createNew()

	def createNew(self):
		self.tree.DeleteChildren(self.tree.treeroot)
		self.file=''
		self.curShader			= None
		self.curObj				= None
		self.SetTitle('Bagisas v'+VERSION)
		self.setChanged(0)
		self.shaderp.reset()
		self.lightp.reset()
		self.treep.reset()
		self.notebook.SetSelection(0)
			
	def onSaveShader(self, evt):
		shader, name = self.shaderp.getSource()
		wildcard = 'Shader Files(*.mat)|*.mat|XML Files(*.xml)|*.xml|All files (*.*)|*.*'
		dlg = wxFileDialog(self, 'Save Shader', self.dirname, name, wildcard, wxSAVE|wxOVERWRITE_PROMPT)
		if dlg.ShowModal() == wxID_OK:
			try:
				of = open(dlg.GetPath(), 'w')
				print >> of, shader
				of.close
			except:
				type, val, tb = sys.exc_info()
				k = traceback.format_tb(tb, None) + traceback.format_exception_only(type, val)
				traceo = string.join([string.lstrip(l) for l in k])
				edlg = wxMessageDialog(self,
									  'An error occured writing the shadersource!\n'+traceo,
									  'ERROR!',
									  wxOK | wxICON_ERROR)
				ok = edlg.ShowModal()
				
	def onSave(self, evt):
		if self.file != '':
			self.tree.save(os.path.join(self.dirname, self.file))
		else:
			self.onSaveAs(evt)
			
	def onSaveAs(self, evt):
		wildcard = 'XML Files(*.xml)|*.xml|All files (*.*)|*.*'
		dlg = wxFileDialog(self, 'Save As...', self.dirname, self.file, wildcard, wxSAVE|wxOVERWRITE_PROMPT)
		if dlg.ShowModal() == wxID_OK:
			self.tree.save(dlg.GetPath())
			self.dirname = os.path.dirname(dlg.GetPath())
			self.file	 = os.path.basename(dlg.GetPath())
			self.setChanged(0)
	
	def getShaderObj(self, root, shader, cookie=0):
		'''Find correspondending object-node for the given shader'''
		if self.tree.ItemHasChildren(root):
			child , nco = self.tree.GetFirstChild(root, cookie)
			for n in xrange ( self.tree.GetChildrenCount ( root, 0 ) ):
				data = self.tree.GetPyData(child)
				if data['name'] == u'object':
					sn = data['attributes'].get(u'shader_name', '')
					if sn == shader:
						return child
				else:
					rv = self.getShaderObj(child, shader, nco)
					if rv:
						return rv
				child = self.tree.GetNextSibling(child)
		else:
			return None

	def onCopyToShader(self, evt):
		'''Handles the menu event: Copy To Shaderpanel'''
		dummy = self.copyToShader()

	def copyToShader(self):
		'''Copy selected shader to shaderpanel'''
		if self.tree.GetPyData(self.tree.GetSelection()).get('name', '') == u'shader':
			self.curShader = self.tree.GetSelection()
		else:
			shader	= self.traverseTo(self.tree.GetSelection(), u'shader')
			if shader:
				self.curShader = shader
			else:
				self.curShader = None
				
		if self.curShader:
			sn =  self.tree.GetPyData(self.curShader).get(u'attributes', '').get(u'name', '')
			if sn != '':
				self.curObj = self.getShaderObj(self.tree.treeroot, sn)
			st = string.lower(self.tree.GetPyData(self.curShader).get(u'attributes', '').get(u'type', ''))
			if st == 'generic':
				self.shaderp.clearValues()
				self.shaderp.setValues(self.curShader, self.curObj)
				self.notebook.SetSelection(1)
				return 1
			else:
				return 0
		else:
			return 0

	def onCopyToLight(self, evt):
		'''Handles the menu event: Copy To Lightpanel'''
		dummy = self.copyToLight

	def copyToLight(self):
		'''Copy selected light to lightbuttons'''
		if self.tree.GetPyData(self.tree.GetSelection()).get('name', '') == u'light':
			self.curLight = self.tree.GetSelection()
		else:
			light	= self.traverseTo(self.tree.GetSelection(), u'light')
			if light:
				self.curLight = light
			else:
				self.curLight = None
				
		if self.curLight:
			self.lightp.setValues(self.curLight)
			self.notebook.SetSelection(2)
			return 1
		else:
			return 0

	def traverseTo(self, node, name):
		'''Detraverse(?) tree, to see if node is a child of a shader-node'''
		if node == self.tree.treeroot:
			return None
		else:
			if self.tree.GetPyData(self.tree.GetItemParent(node)).get('name', '') == name:
				return self.tree.GetItemParent(node)
			else:
				return self.traverseTo(self.tree.GetItemParent(node), name)
			
	def onOpen(self, evt):
		''' Open A File'''
		wildcard = 'XML Files(*.xml)|*.xml|All files (*.*)|*.*'
		dlg = wxFileDialog(self, 'Choose a file', self.dirname, '', wildcard, wxOPEN|wxFILE_MUST_EXIST)
		
		if dlg.ShowModal() == wxID_OK:
			self.file=os.path.basename(dlg.GetPath())
			self.dirname=os.path.dirname(dlg.GetPath())
			self.treep.attrgrid.ClearGrid()
			self.tree.DeleteAllItems()
			self.SetTitle('Bagisas v'+VERSION+' '+self.file)
			self.tree.createDOMTree()
			self.tree.Expand(self.tree.GetRootItem())
		dlg.Destroy()
		
	def onClose(self, evt):
		''' Ends The Application'''
		if self.changed:
			dlg = wxMessageDialog(self, 'There are unsaved changes. Do you really want to end?', 'Exit Program?',
								  wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION) 
			if dlg.ShowModal() == wxID_YES: 
				print 'Byebye!'
				self.removeThumbs()
				while not self.Destroy():
					print 'Please wait.'
			else:
				dlg.Destroy()
		else:
			print 'Byebye!'
			self.removeThumbs()
			while not self.Destroy():
				print 'Please wait.'
			
	def onExit(self, evt):
		self.Close(1) 
		
	def removeThumbs(self):
		for img in self.remove:
			file = os.path.splitext(os.path.basename(img))[0]
			if os.path.exists(file+'.jpg'):
				os.remove(file+'.jpg')
			if os.path.exists(file+'.tga'):
				os.remove(file+'.tga')
		
	def onPrefs(self, evt):
		''' Show Preferences-Dialog'''
		dlg = PrefsDialog(self, self, -1, "Preferences", size=wxSize(500, 300),
					   style = wxCAPTION | wxSYSTEM_MENU)
		val = dlg.ShowModal()
		if val == wxID_OK:
			self.config['render.yafex']	 = dlg.getYafEx()
			self.config['render.numcpu'] = dlg.getNumCpu()
			self.config['render.netop']	 = dlg.getNetOp()
			self.config['render.scedir'] = dlg.getSceDir()
			if self.file == '':
				self.dirname        = self.config['render.scedir']

			
	def onSavePrefs(self, evt):
		''' Save Preferences Wrapper'''
		self.savePrefs()
		
	def savePrefs(self):
		''' Save Preferences '''
		if os.name == 'nt' or os.name == 'dos':
			fo = open('bagisas.ini', 'w')
		else:
			fo = open(os.path.expanduser('~/.bagisasrc'), 'w')
		cp = ConfigParser.ConfigParser(self.config)
		cp.write(fo)

	def onAbout(self, evt):
		''' Show About-Dialog'''
		d= wxMessageDialog( self, ' A YAFRAY browser \n'
							' in wxPython','About Bagisas', wxOK)
		d.ShowModal() 
		d.Destroy() 
		
	def onRender(self, evt):
		''' Renders The Current File'''
		try:
			if not self.changed:
				if not self.render or not self.render.isAlive():
					self.render = Render.Render(self.treep,
												self.treep.output,
												self.treep.preview,
												os.path.join(self.dirname, self.file),
												self.outfile, 0, 1)
					self.render.start()
			else:
				add = time.time()
				tempfn = os.path.join(self.dirname,'tmp%f.xml' % (add))
				self.tree.save(tempfn, 0)
				if not self.render or not self.render.isAlive():
					self.render = Render.Render(self.treep,
												self.treep.output,
												self.treep.preview,
												tempfn, self.outfile, 1, 1)
					self.render.start()
		except:
			dlg = wxMessageDialog(self,
								  'An error has occured during rendering!'+self.getTraceback(),
								  'ERROR!',
								  wxOK | wxICON_ERROR)
			ok = dlg.ShowModal()

			
	def onPreview(self, evt):
		''' Previews The Current File'''
		dlg = PreviewDialog(self, self, -1, "Preview Settings", size=wxSize(500, 300),
							style = wxCAPTION | wxSYSTEM_MENU)
		val = dlg.ShowModal()
		if val == wxID_OK:
			filename = os.path.join(self.dirname,'BScePre.xml')
			outfile = open(filename, 'w')
	
			print >> outfile, '<scene>'

			if dlg.getDefShader():
				cv = dlg.getShader()
				print >> outfile, ' <shader type = "generic" name = "default">'
				print >> outfile, '  <attributes>'
				print >> outfile, '   <color r="%f" g="%f" b="%f" />' % (cv[0]/255.0, cv[1]/255.0, cv[2]/255.0)
				print >> outfile, '  </attributes>'
				print >> outfile, ' </shader>'
				transforms = self.tree.getNodes('transform', self.tree.treeroot, 0, [])
				objects = self.tree.getNodes('object', self.tree.treeroot, 0, [])
				for t in transforms:
					objects.append(t)
				self.writePreviewBranch(outfile, objects, replace = 'shader_name', with = 'default')
			else:
				shaders = self.tree.getNodes('shader', self.tree.treeroot, 0, [])
				self.writePreviewBranch(outfile, shaders)
				transforms = self.tree.getNodes('transform', self.tree.treeroot, 0, [])
				objects = self.tree.getNodes('object', self.tree.treeroot, 0, [])
				for t in transforms:
					objects.append(t)
				self.writePreviewBranch(outfile, objects)

			cam = self.tree.getNodes('camera', self.tree.treeroot, 0, [])
			if dlg.getCamLight():
				loc = self.tree.getNodes('from', cam[-1], 0, [])[-1]
				x = self.tree.GetPyData(loc)['attributes'][u'x']
				y = self.tree.GetPyData(loc)['attributes'][u'y']
				z = self.tree.GetPyData(loc)['attributes'][u'z']
				print >> outfile, ' <light name="light1" power="%s" type="pointlight">' % (dlg.getLightPower())
				print >> outfile, '  <from x="%s" y="%s" z="%s" />' % (x, y, z)
				print >> outfile, '  <color r="1" g="1" b="1" />'
				print >> outfile, ' </light>'
			else:
				lights = self.tree.getNodes('light', self.tree.treeroot, 0, [])
				self.writePreviewBranch(outfile, lights)
				
			if dlg.getSize():
				self.writePreviewBranch(outfile, cam, replace = ['resx', 'resy'], with = [dlg.getResX(), dlg.getResY()])
			else:
				self.writePreviewBranch(outfile, cam)
			render = self.tree.getNodes('render', self.tree.treeroot, 0, [])
			self.writePreviewBranch(outfile, render, replace = ['samples', 'AA_passes'], with = dlg.getAntiAlias())
			name =  self.tree.GetPyData(render[-1])['attributes'].get('outfile', 'salida.tga')
			print >> outfile, '</scene>'
			outfile.close()

			if not self.render or not self.render.isAlive():
				self.render = Render.Render(self.treep,
											self.treep.output,
											self.treep.preview,
											filename,
											self.outfile, 0, 1)
				self.render.start()
			else:
				print 'New thread blocked!'
			
	def writePreviewBranch(self, outfile, elemlist, indent = ' ', replace = '', with = ''):
		for elem in elemlist:
			data = self.tree.GetPyData(elem)
			name = data['name']
			node = indent + '<%s ' % (name)
			keys = data['attributes'].keys()
			keys.sort()
			for d in keys:
				if type(replace) == StringType:
					if d == replace:
						node += '%s="%s" ' % (d, with)
					else:
						node += '%s="%s" ' % (d, data['attributes'][d])
				elif type(replace) == ListType:
					if type(with) != ListType:
						if d in replace:
							node += '%s="%s" ' % (d, with)
						else:
							node += '%s="%s" ' % (d, data['attributes'][d])
					else:
						if d in replace:
							node += '%s="%s" ' % (d, with[replace.index(d)])
						else:
							node += '%s="%s" ' % (d, data['attributes'][d])
					
			string.rstrip(node)
			
			if name in SELFCLOSING:
				print >> outfile, '%s' % (node+' />')
			else:
				print >> outfile, '%s' % (node+'>')
				self.writePreviewBranch(outfile, self.tree.getNodes('', elem, 0, []), indent+' ', replace, with)
				print >> outfile, '%s</%s>' % (indent, name)
		
	def getTraceback(self):
		type, val, tb = sys.exc_info()
		k = traceback.format_tb(tb, None) + traceback.format_exception_only(type, val)
		return string.join([string.lstrip(l) for l in k])

	def onStopThread(self, evt):
		self.render.join()
		
	def onThread(self, evt):
		print '------------------------------------------------------------'
		print threading.activeCount()
		print self.render.getName()
		print self.render.isAlive()
		
	#def onSizePreview(self, evt):
	#	print evt.GetSize()

	def attribToDict(self, attrib):
		rv = {}
		x = 0
		for x in attrib.getNames():
			rv[x] = attrib.getValue(x)
		return rv
			
	def setChanged(self, change):
		if change and not self.changed:
			self.SetTitle('Bagisas v'+VERSION+' '+self.file+' [*]')
		if not change and self.changed:
			self.SetTitle('Bagisas v'+VERSION+' '+self.file)
		self.changed = change

	def onTestYaf(self, evt):
		outp = ''
		cmd = self.config['render.yafex']
		yafouterr, yafin = popen2.popen4(cmd, -1, 'b')

		outerr = yafouterr.read(1)
		while outerr:
			outp += outerr
			outerr = yafouterr.read(1)

		ec1 = yafouterr.close()
		ec2 = yafin.close()

		d= wxMessageDialog( self, outp,'Yafray Test', wxOK)
		d.ShowModal() 
		d.Destroy() 

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		

def main(file=''):
	app = wxPySimpleApp()
	frame = Bagisas(None, -1, 'Bagisas v'+VERSION+' '+file, file)
	app.MainLoop()
	
if __name__ == '__main__':
	if len(sys.argv) > 1:
		#profile.run('main(sys.argv[1])')
		main(sys.argv[1])
	else:
		main()


## History:
##
##  v 0.1.0 21.Jul.2003 ('I Wanna Be Somebody'):
##    - XML parsing is now faster (switched from DOM to SAX)
##    - Doubleclick on shader/light-branch switches to shader-/lightpanel
##    - Scene directory preference setting
##    - FIXED: AttribGrid doesn't handle all keys
##    - FIXED: included more self-closing tags
##    - FIXED: UV-mapping don't get messed up when modifing shaders
## 
##  v 0.0.6 24.Jun.2003 ('Shoplifter'):
##    - Help -> Test Yafray
##    - Preview Render
##
##  v 0.0.5 23.May.2003 ('Zombie Eaters'):
##    - Lightpanel
##    - Changes In Table Are Transfered To Shaderpanel
##    - Save Shadersource
##    - wxTELETYPE For Renderlogs
##    - Preview-Window/Click in Shaderpanel and Treepanel
##    - Save/Clear Renderlog
##    - Colored Togglebuttons (MSW)
##    - Lowered Power Of Arealight In Shaderpanel Preview
##
##  v 0.0.4 28.Apr.2003 ('Mr.Bojangles'):
##    - Colordialog Ends Editing Of Gridcell
##    - Switched to Notebookstyle
##    - Shaderpanel
##    - "Improved" Error Messages
##    - FIXED: PIL-Bug: Sometimes The Rendered Images Won't Show
##      
##  v 0.0.3 27.Mar.2003 ('Sign Of The Wolf'):
##    - Edit Colors With Colordialog
##    - Improved Output Of Renderthread
##    - Rendered Image Fits To Preview
##    - Restructured Some Code
##    - FIXED: Outfile Element (-> Yable) Sets The Outfile Value
##
##  v 0.0.2 17.Mar.2003 ('See Dem Coming'):
##    - Tree/Node Expand/Collapse
##    - View rendered image
##    - Preferences Dialog
##    - shader name changed => automagically change object-attribute shader_name
##    - FIXED: Attribute Changes Weren't Recognized Until Save
##    - FIXED: outfile change resulted in not viewing the image
##
##  v 0.0.1 13.Mar.2003 ('King Kong Five'):
##    - Inital Release
##
## TODO:
##  (In no particular order)
##
##  - Filterpanel
##  - Exceptionhandling in Shaderpanel when writing file
##  - Shader-Library
##  - Comments!!
##  - Add items
##  - Kill render-thread
##  - Undo (??)
##  - Show all attributes of an element, even those optional which are not in the file
##  - MDI w/ Cut/Copy & Paste
##  - Text editor
##  - XML-View of the shader
