import os.path, string
from wxPython.wx import wxFrame, wxBoxSizer, wxDialog, wxSizer, wxStaticText
from wxPython.wx import wxTextCtrl, wxStaticLine, wxButton, wxNewId, wxPoint
from wxPython.wx import wxSlider, wxCheckBox, EVT_BUTTON, wxFileDialog
from wxPython.wxc import *
from wxPython.lib.buttons import *
from wxPython.lib.evtmgr import eventManager

ID_BTN_YAFEX = wxNewId()
ID_BTN_SCEDIR = wxNewId()
ID_BTN_COLOR = wxNewId()

class PrefsDialog(wxDialog):
	
	def __init__(self, parent, *args, **kwargs):
		apply(wxDialog.__init__,(self,) + args, kwargs)
		self.parent = parent
		sizer = wxBoxSizer(wxVERTICAL)
		label = wxStaticText(self, -1, 'Yafray Options')
		sizer.Add(label, 0, wxALIGN_LEFT|wxALL, 5)

		box = wxBoxSizer(wxHORIZONTAL)
		label = wxStaticText(self, -1, 'Yafray Executable:')
		box.Add(label, 0, wxALIGN_RIGHT|wxALL, 3)
		self.yafex = wxTextCtrl(self, -1, self.parent.config['render.yafex'], size=(120,-1))
		box.Add(self.yafex, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3)
		btn1 = wxButton(self, ID_BTN_YAFEX , 'Browse...', wxPoint(0,0), style = wxBU_EXACTFIT)
		box.Add(btn1, 1, wxALIGN_RIGHT|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		box = wxBoxSizer(wxHORIZONTAL)
		label = wxStaticText(self, -1, 'Number Of CPUs To Use:')
		box.Add(label, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 3)
		self.slid = wxSlider(self, -1, int(self.parent.config['render.numcpu']),
							 1, 16, style = wxSL_HORIZONTAL|wxSL_AUTOTICKS|wxSL_LABELS)
		box.Add(self.slid, 1, wxALIGN_CENTRE|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		box = wxBoxSizer(wxHORIZONTAL)
		label = wxStaticText(self, -1, 'Use Net Optimized:')
		box.Add(label, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 3)
		self.cb = wxCheckBox(self, -1, '')
		self.cb.SetValue(int(self.parent.config['render.netop']))
		box.Add(self.cb, 1, wxALIGN_CENTRE|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		line = wxStaticLine(self, -1, size=(20,-1), style=wxLI_HORIZONTAL)
		sizer.Add(line, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP, 3)

		box = wxBoxSizer(wxHORIZONTAL)
		label = wxStaticText(self, -1, 'Scene Directory:')
		box.Add(label, 0, wxALIGN_RIGHT|wxALL, 3)
		self.scedir = wxTextCtrl(self, -1, self.parent.config['render.scedir'], size=(120,-1))
		box.Add(self.scedir, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3)
		btn2 = wxButton(self, ID_BTN_SCEDIR , 'Browse...', wxPoint(0,0), style = wxBU_EXACTFIT)
		box.Add(btn2, 1, wxALIGN_RIGHT|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		line = wxStaticLine(self, -1, size=(20,-1), style=wxLI_HORIZONTAL)
		sizer.Add(line, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP, 3)

		box = wxBoxSizer(wxHORIZONTAL)
		btn = wxButton(self, wxID_OK, ' OK ')
		btn.SetDefault()
		box.Add(btn, 0, wxALIGN_CENTRE|wxALL, 5)
		btn = wxButton(self, wxID_CANCEL, ' Cancel ')
		box.Add(btn, 0, wxALIGN_CENTRE|wxALL, 5)
		sizer.AddSizer(box, 0, wxALIGN_CENTER_VERTICAL|wxALIGN_CENTER_HORIZONTAL|wxALL, 5)

		self.SetSizer(sizer)
		self.SetAutoLayout(1)
		sizer.Fit(self)

		eventManager.Register(self.onBrowse, EVT_BUTTON, win=self, id=ID_BTN_YAFEX)
		eventManager.Register(self.onBrowse, EVT_BUTTON, win=self, id=ID_BTN_SCEDIR)

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def onBrowse(self, evt):
		if evt.GetId() == ID_BTN_YAFEX:
			dlg = wxFileDialog(self, 'Choose Yafray binary', '.', '',
							   'Yafray Binary (yafray.exe)|yafray.exe|All Files (*.*)|*.*', wxOPEN)
			try:
				if dlg.ShowModal() == wxID_OK:
					bin = os.path.join(dlg.GetDirectory(),dlg.GetFilename())
					self.yafex.SetValue(bin)
			finally:
				dlg.Destroy()

		if evt.GetId() == ID_BTN_SCEDIR:
			dlg = wxDirDialog(self, 'Choose scene directory', self.parent.config['render.scedir'])
			try:
				if dlg.ShowModal() == wxID_OK:
					self.scedir.SetValue(dlg.GetPath())
			finally:
				dlg.Destroy()

	def getYafEx(self):
		return self.yafex.GetValue()

	def getSceDir(self):
		return self.scedir.GetValue()

	def getNumCpu(self):
		return self.slid.GetValue()

	def getNetOp(self):
		return self.cb.GetValue()

class PreviewDialog(wxDialog):
	
	def __init__(self, parent, *args, **kwargs):
		apply(wxDialog.__init__,(self,) + args, kwargs)
		self.parent = parent
		sizer = wxBoxSizer(wxVERTICAL)

		# Default Shader?
		box = wxBoxSizer(wxHORIZONTAL)
		self.cbShader = wxCheckBox(self, -1, 'Use Default Shader')
		self.cbShader.SetValue(int(self.parent.config['preview.defshader']))
		box.Add(self.cbShader, 1, wxALIGN_RIGHT|wxALL, 3)
		self.btnColor = wxGenButton(self, ID_BTN_COLOR, '')
		cv = map(int, string.split(self.parent.config['preview.shader'][1:-1], ','))
		self.btnColor.SetBackgroundColour(wxColour(cv[0], cv[1], cv[2]))
		box.Add(self.btnColor, 1, wxALIGN_RIGHT|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		# Camera Light?
		box = wxBoxSizer(wxHORIZONTAL)
		self.cbCamLight = wxCheckBox(self, -1, 'Use Camera Light')
		self.cbCamLight.SetValue(int(self.parent.config['preview.camlight']))
		box.Add(self.cbCamLight, 1, wxALIGN_RIGHT|wxALL, 3)
		self.spLPow = wxSpinCtrl(self, -1, min=1, max=2500, initial=int(self.parent.config['preview.lightpower']))
		dum = self.spLPow.GetValue()
		self.spLPow.SetValue(int(self.parent.config['preview.lightpower']))
		dum = self.spLPow.GetValue()
		box.Add(self.spLPow, 1, wxALIGN_RIGHT|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		# Anti Aliasing?
		box = wxBoxSizer(wxHORIZONTAL)
		box.Add(wxStaticText(self, -1, 'Anti Aliasing:'), 1, wxALIGN_RIGHT|wxALL, 3)
		self.spAnti = wxSpinCtrl(self, -1, min=1, max=100, initial=int(self.parent.config['preview.antialias']))
		dum = self.spAnti.GetValue()
		self.spAnti.SetValue(int(self.parent.config['preview.antialias']))
		dum = self.spAnti.GetValue()
		box.Add(self.spAnti, 1, wxALIGN_RIGHT|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		# Size?
		box = wxBoxSizer(wxHORIZONTAL)
		self.cbSize = wxCheckBox(self, -1, 'Size')
		self.cbSize.SetValue(int(self.parent.config['preview.size']))
		box.Add(self.cbSize, 1, wxALIGN_RIGHT|wxALL, 3)

		self.spResX = wxSpinCtrl(self, -1, min=1, max=2500, initial=int(self.parent.config['preview.sizerx']))
		dum = self.spResX.GetValue()
		self.spResX.SetValue(int(self.parent.config['preview.sizerx']))
		dum = self.spResX.GetValue()
		box.Add(self.spResX, 0, wxALIGN_RIGHT|wxALL, 3)
		box.Add(wxStaticText(self, -1, 'x'), 0, wxALIGN_CENTER|wxALL, 3)
		self.spResY = wxSpinCtrl(self, -1, min=1, max=2500, initial=int(self.parent.config['preview.sizery']))
		dum = self.spResY.GetValue()
		self.spResY.SetValue(int(self.parent.config['preview.sizery']))
		dum = self.spResY.GetValue()
		box.Add(self.spResY, 0, wxALIGN_RIGHT|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		line = wxStaticLine(self, -1, size=(20,-1), style=wxLI_HORIZONTAL)
		sizer.Add(line, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP, 3)

		box = wxBoxSizer(wxHORIZONTAL)
		btn = wxButton(self, wxID_OK, 'Preview')
		btn.SetDefault()
		box.Add(btn, 0, wxALIGN_CENTRE|wxALL, 5)
		self.btnSave = wxButton(self, wxID_SAVE, ' Save ')
		box.Add(self.btnSave, 0, wxALIGN_CENTRE|wxALL, 5)
		btn = wxButton(self, wxID_CANCEL, ' Cancel ')
		box.Add(btn, 0, wxALIGN_CENTRE|wxALL, 5)
		sizer.AddSizer(box, 0, wxALIGN_CENTER_VERTICAL|wxALIGN_CENTER_HORIZONTAL|wxALL, 5)

		self.SetSizer(sizer)
		self.SetAutoLayout(1)
		sizer.Fit(self)

		eventManager.Register(self.onColor,  EVT_BUTTON, win=self, id=ID_BTN_COLOR)
		eventManager.Register(self.onSave,   EVT_BUTTON, win=self, id=wxID_SAVE)

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def onColor(self, evt):
		colordata = wxColourData()
		color = self.btnColor.GetBackgroundColour()
		colordata.SetColour(color)
		colordata.SetChooseFull(1)
		dlg = wxColourDialog(self, colordata)
		if dlg.ShowModal() == wxID_OK:
			newcol = dlg.GetColourData()
			self.btnColor.SetBackgroundColour(newcol.GetColour())
			
	def onSave(self, evt):
		'''Save Preview Settings'''
		self.parent.config['preview.defshader'] = self.cbShader.GetValue()
		col = self.btnColor.GetBackgroundColour()
		shader = '(%s, %s, %s)' % (col.Red(), col.Green(), col.Blue())
		self.parent.config['preview.shader']     = shader
		self.parent.config['preview.camlight']	 = self.cbCamLight.GetValue()
		self.parent.config['preview.lightpower'] = self.spLPow.GetValue()
		self.parent.config['preview.antialias']  = self.spAnti.GetValue()
		self.parent.config['preview.size']  = self.cbSize.GetValue()
		self.parent.config['preview.sizerx']  = self.spResX.GetValue()
		self.parent.config['preview.sizery']  = self.spResY.GetValue()
		self.parent.savePrefs()
		
	def getDefShader(self):
		return self.cbShader.GetValue()

	def getShader(self):
		col = self.btnColor.GetBackgroundColour()
		return (col.Red(), col.Green(), col.Blue())

	def getCamLight(self):
		return self.cbCamLight.GetValue()

	def getLightPower(self):
		return self.spLPow.GetValue()

	def getAntiAlias(self):
		return self.spAnti.GetValue()

	def getSize(self):
		return self.cbSize.GetValue()

	def getResX(self):
		return self.spResX.GetValue()

	def getResY(self):
		return self.spResY.GetValue()
