import string
from NumValidator import *
from wxPython.wx import *
from wxPython.lib.evtmgr import eventManager
from wxPython.lib.rcsizer import RowColSizer
from wxPython.lib.colourchooser import wxPyColourChooser

ID_PREVIEW	  = wxNewId()

class LightPanel(wxPanel):

	def __init__(self, master, parent, id):
		wxPanel.__init__(self,
						 parent,
						 id,
						 size = parent.GetClientSize(),
						 name='Tree')

		self.master = master
		self.config = self.master.config
		self.render = None
		self.size = 240

		types = [x+'light' for x in ['point', 'sun', 'soft', 'spot', 'photon', 'area', 'hemi', 'path']]

		# Common members for all types
		# Action buttons
		pAction	  = wxPanel(self, -1, size=(250,25))
		self.bAdd = wxButton(pAction, -1, 'Add To Tree')
		self.bCopy = wxButton(pAction, -1, 'Copy To Tree')
		self.layItOut(pAction, [self.bAdd, self.bCopy])
		self.bCopy.Disable()
		
		# Reference
		pReference = wxPanel(self, -1)
		self.stReference = wxStaticText(pReference, -1, 'No Reference', style=wxALIGN_CENTRE|wxST_NO_AUTORESIZE)
		self.bReset = wxButton(pReference, -1, 'Reset')
		self.layItOut(pReference, [self.bReset, self.stReference])
							  
		# Name
		pName	= wxPanel(self, -1)
		self.txtName = wxTextCtrl(pName, -1, 'default')
		self.layItOut(pName, [wxStaticText(pName, -1, 'Name: '), self.txtName])

		# Type
		pType	= wxPanel(self, -1)
		self.coiType = wxChoice(pType, -1, wxDefaultPosition, wxDefaultSize, types)
		self.layItOut(pType, [wxStaticText(pType, -1, 'Type: '), self.coiType])

		# Power
		pPower	= wxPanel(self, -1, size=(200,20))
		self.txtPower = wxTextCtrl(pPower, -1, '', validator = NumValidator())
		self.layItOut(pPower, [wxStaticText(pPower, -1, 'Power: '), self.txtPower])

		# Color
		pColor	  = wxPanel(self, -1, size=(200,20))
		self.bColor = wxButton(pColor, -1, '  ')
		self.layItOut(pColor, [wxStaticText(pColor, -1, 'Color'), self.bColor])
		
		self.common = self.GetChildren()

		# Sizer
		self.rcs = RowColSizer()

		self.rcs.Add(pAction, row=0, col=0, flag=wxEXPAND)
		self.rcs.Add(pReference, row=1, col=0, flag=wxEXPAND)
		self.rcs.Add(pType, row=2, col=0, flag=wxEXPAND)
		self.rcs.Add(pName, row=3, col=0, flag=wxEXPAND)
		self.rcs.Add(pPower, row=4, col=0, flag=wxEXPAND)
		self.rcs.Add(pColor, row=5, col=0, flag=wxEXPAND)

		self.initalizeValues()
		self.setPointSunLayout()

		self.SetAutoLayout(1)
		self.SetSizer(self.rcs)
		self.Layout()

		self.setEventHandling()

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def setEventHandling(self):
		''' Connects GUI objects with events'''
		eventManager.Register(self.onCopyToTree,  EVT_BUTTON, self.bCopy)
		eventManager.Register(self.onAddToTree,   EVT_BUTTON, self.bAdd)
		eventManager.Register(self.onColor,       EVT_BUTTON, self.bColor)
		eventManager.Register(self.onReset,       EVT_BUTTON, self.bReset)
		eventManager.Register(self.onChoice,      EVT_CHOICE, self.coiType)

	def initalizeValues(self):
		node = self.master.curLight
		if node:
			val = self.getNodeValue(node, 'from', 'Coord')
			if val:
				self.txtNameFXV = '%s' % val[0]
				self.txtNameFYV = '%s' % val[1]
				self.txtNameFZV = '%s' % val[2]
			val = self.getNodeValue(node, 'to', 'Coord')
			if val:
				self.txtNameTXV = '%s' % val[0]
				self.txtNameTYV = '%s' % val[1]
				self.txtNameTZV = '%s' % val[2]
			val = self.getNodeValue(node, 'a', 'Coord')
			if val:
				self.txtNameAXV = '%s' % val[0]
				self.txtNameAYV = '%s' % val[1]
				self.txtNameAZV = '%s' % val[2]
			val = self.getNodeValue(node, 'b', 'Coord')
			if val:
				self.txtNameBXV = '%s' % val[0]
				self.txtNameBYV = '%s' % val[1]
				self.txtNameBZV = '%s' % val[2]
			val = self.getNodeValue(node, 'c', 'Coord')
			if val:
				self.txtNameCXV = '%s' % val[0]
				self.txtNameCYV = '%s' % val[1]
				self.txtNameCZV = '%s' % val[2]
			val = self.getNodeValue(node, 'd', 'Coord')
			if val:
				self.txtNameDXV = '%s' % val[0]
				self.txtNameDYV = '%s' % val[1]
				self.txtNameDZV = '%s' % val[2]
			self.txtResV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'res', '')
			self.txtRadiusV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'radius', '')
			self.txtBiasV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'bias', '')
			self.txtSizeV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'size', '')
			self.txtBlendV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'blend', '')
			self.txtBeamFalloffV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'beam_falloff', '')

			ha = string.strip(string.lower(self.master.tree.GetPyData(node).get('attributes', {}).get(u'halo', 'off')))
			if ha == 'on':
				self.cbHaloV = 1
			else:
				self.cbHaloV = 0
			self.txtBlurV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'blur', '')
			self.txtFogDensityV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'fog_density', '')
			self.txtPhotonsV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'photons', '')
			self.txtDepthV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'depth', '')
			self.txtSearchV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'search', '')
			self.txtAngleV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'Angle', '')
			self.txtFixedradiusV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'fixedradius', '')
			self.txtClusterV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'cluster', '')
			self.txtSamplesV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'samples', '')
			self.txtPSamplesV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'psamples', '')
			self.coiModeV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'mode', '')
			self.txtCacheSizeV = self.master.tree.GetPyData(node).get('attributes', {}).get(u'cache_size', '')
			cs = string.strip(string.lower(self.master.tree.GetPyData(node).get('attributes', {}).get(u'cast_shadows', 'off')))
			if cs == 'on':
				self.cbCShadowV = 1
			else:
				self.cbCShadowV = 0
			rs = string.strip(string.lower(self.master.tree.GetPyData(node).get('attributes', {}).get(u'randomsampling', 'off')))
			if rs == 'on':
				self.cbRandomsamplingV = 1
			else:
				self.cbRandomsamplingV = 0
			mc = string.strip(string.lower(self.master.tree.GetPyData(node).get('attributes', {}).get(u'use_QMC', 'off')))
			if mc == 'on':
				self.cbQMCV = 1
			else:
				self.cbQMCV = 0
			ca = string.strip(string.lower(self.master.tree.GetPyData(node).get('attributes', {}).get(u'use_QMC', 'off')))
			if ca == 'on':
				self.cbCacheV = 1
			else:
				self.cbCacheV = 0
		else:
			self.clearValues()
			
	def clearValues(self):
		self.txtNameFXV = self.txtNameFYV =	self.txtNameFZV = ''
		self.txtNameTXV = self.txtNameTYV =	self.txtNameTZV = ''
		self.txtNameAXV = self.txtNameAYV =	self.txtNameAZV = ''
		self.txtNameBXV = self.txtNameBYV =	self.txtNameBZV = ''
		self.txtNameCXV = self.txtNameCYV =	self.txtNameCZV = ''
		self.txtNameDXV = self.txtNameDYV =	self.txtNameDZV = ''
		self.txtResV = self.txtRadiusV = self.txtBiasV = self.txtSizeV = ''
		self.txtBlendV = self.txtBeamFalloffV = self.txtPhotonsV = self.txtDepthV = ''
		self.txtSearchV = self.txtAngleV = self.txtFixedradiusV = self.txtClusterV = ''
		self.txtSamplesV = self.txtPSamplesV = self.coiModeV = self.txtCacheSizeV = ''
		self.cbRandomsamplingV = 0
		self.cbCShadowV = 0
		self.txtBlurV   = self.txtFogDensityV = ''
		self.cbHaloV = 0
		self.cbQMCV = 0
		self.cbCacheV = 0

	def onChoice(self, evt):
		type = string.strip(string.lower(self.coiType.GetStringSelection()))
		if  type == 'pointlight' or type == 'sunlight':
			self.setPointSunLayout()
		elif type == 'softlight':
			self.setSoftLayout()
		elif type == 'spotlight':
			self.setSpotLayout()
		elif type == 'photonlight':
			self.setPhotonLayout()
		elif type == 'hemilight':
			self.setHemiLayout()
		elif type == 'pathlight':
			self.setPathLayout()
		elif type == 'arealight':
			self.setAreaLayout()
		else:
			print type
		
	def onReset(self, evt):
		self.reset()
		
	def reset(self):
		self.stReference.SetLabel('No Reference')
		self.bCopy.Enable(0)
		self.clearValues()
		self.master.curLight = None
		self.coiType.SetSelection(0)
		self.setPointSunLayout()

	def onColor(self, evt):
		dlg = wxColourDialog(self.master)
		dlg.GetColourData().SetChooseFull(1)
		if dlg.ShowModal() == wxID_OK:
			data = dlg.GetColourData()
			evt.GetEventObject().SetBackgroundColour(data.GetColour())
			evt.GetEventObject().Refresh()
		
	def layItOut(self, panel, items, weight=[]):
		sizer = wxBoxSizer(wxHORIZONTAL)
		if not weight:
			weight = len(items)*[1]
		c = 0
		for x in items:
			sizer.Add(x, weight[c], wxEXPAND)
			c += 1
		panel.SetAutoLayout(1)
		panel.SetSizer(sizer)
		panel.Layout()
	
	def setPointSunLayout(self):
		self.clearSizer()
		# Cast Shadow
		pCShadow	= wxPanel(self, -1)
		self.cbCShadow = wxCheckBox(pCShadow, -1, '')
		self.cbCShadow.SetValue(self.cbCShadowV)
		self.layItOut(pCShadow, [wxStaticText(pCShadow, -1, 'Cast Shadows:'), self.cbCShadow])
		# From
		pFrom, pFromXYZ	= self.locationPanels('From', ['self.txtNameFX', 'self.txtNameFY', 'self.txtNameFZ'],
											  ['self.txtNameFXV', 'self.txtNameFYV', 'self.txtNameFZV'])

		self.addToSizer([pCShadow, pFrom, pFromXYZ])

	def setSoftLayout(self):
		self.clearSizer()
		# From
		pFrom, pFromXYZ	= self.locationPanels('From', ['self.txtNameFX', 'self.txtNameFY', 'self.txtNameFZ'],
											  ['self.txtNameFXV', 'self.txtNameFYV', 'self.txtNameFZV'])
		# Res
		pRes	= wxPanel(self, -1)
		self.txtRes = wxTextCtrl(pRes, -1, self.txtResV, validator = NumValidator())
		self.layItOut(pRes, [wxStaticText(pRes, -1, 'Res: '), self.txtRes])

		# Radius
		pRadius	= wxPanel(self, -1)
		self.txtRadius = wxTextCtrl(pRadius, -1, self.txtRadiusV, validator = NumValidator())
		self.layItOut(pRadius, [wxStaticText(pRadius, -1, 'Radius: '), self.txtRadius])

		# Bias
		pBias	= wxPanel(self, -1)
		self.txtBias = wxTextCtrl(pBias, -1, self.txtBiasV, validator = NumValidator())
		self.layItOut(pBias, [wxStaticText(pBias, -1, 'Bias: '), self.txtBias])

		self.addToSizer([pFrom, pFromXYZ, pRes, pRadius, pBias])

	def setSpotLayout(self):
		self.clearSizer()
		# From
		pFrom, pFromXYZ	= self.locationPanels('From', ['self.txtNameFX', 'self.txtNameFY', 'self.txtNameFZ'],
											  ['self.txtNameFXV', 'self.txtNameFYV', 'self.txtNameFZV'])
		# To
		pTo, pToXYZ	= self.locationPanels('To', ['self.txtNameTX', 'self.txtNameTY', 'self.txtNameTZ'],
										  ['self.txtNameTXV', 'self.txtNameTYV', 'self.txtNameTZV'])

		# Size
		pSize	= wxPanel(self, -1)
		self.txtSize = wxTextCtrl(pSize, -1, self.txtSizeV, validator = NumValidator())
		self.layItOut(pSize, [wxStaticText(pSize, -1, 'Size: '), self.txtSize])

		# Blend
		pBlend	= wxPanel(self, -1)
		self.txtBlend = wxTextCtrl(pBlend, -1, self.txtBlendV, validator = NumValidator())
		self.layItOut(pBlend, [wxStaticText(pBlend, -1, 'Blend: '), self.txtBlend])

		# BeamFalloff
		pBeamFalloff	= wxPanel(self, -1)
		self.txtBeamFalloff = wxTextCtrl(pBeamFalloff, -1, self.txtBeamFalloffV, validator = NumValidator())
		self.layItOut(pBeamFalloff, [wxStaticText(pBeamFalloff, -1, 'BeamFalloff: '), self.txtBeamFalloff])

		# Halo
		pHalo	= wxPanel(self, -1)
		self.cbHalo = wxCheckBox(pHalo, -1, '')
		self.cbHalo.SetValue(self.cbHaloV)
		self.layItOut(pHalo, [wxStaticText(pHalo, -1, 'Halo:'), self.cbHalo])

		# Res
		pRes	= wxPanel(self, -1)
		self.txtRes = wxTextCtrl(pRes, -1, self.txtResV, validator = NumValidator())
		self.layItOut(pRes, [wxStaticText(pRes, -1, 'Res: '), self.txtRes])

		# Blur
		pBlur	= wxPanel(self, -1)
		self.txtBlur = wxTextCtrl(pBlur, -1, self.txtBlurV, validator = NumValidator())
		self.layItOut(pBlur, [wxStaticText(pBlur, -1, 'Blur: '), self.txtBlur])

		# FogDensity
		pFogDensity	= wxPanel(self, -1)
		self.txtFogDensity = wxTextCtrl(pFogDensity, -1, self.txtFogDensityV, validator = NumValidator())
		self.layItOut(pFogDensity, [wxStaticText(pFogDensity, -1, 'Fog Density: '), self.txtFogDensity])

		# Samples
		pSamples	= wxPanel(self, -1)
		self.txtSamples = wxTextCtrl(pSamples, -1, self.txtSamplesV, validator = NumValidator())
		self.layItOut(pSamples, [wxStaticText(pSamples, -1, 'Samples: '), self.txtSamples])

		# Fog
		pFog	  = wxPanel(self, -1, size=(200,20))
		self.bFog = wxButton(pFog, -1, '  ')
		self.layItOut(pFog, [wxStaticText(pFog, -1, 'Fog'), self.bFog])

		fc = self.getNodeValue(self.master.curLight, 'fog')
		if fc: 
			self.bFog.SetBackgroundColour(wxColour(fc[0], fc[1], fc[2]))
			
		eventManager.Register(self.onColor,       EVT_BUTTON, self.bFog)

		self.addToSizer([pFrom, pFromXYZ, pTo, pToXYZ, pSize, pBlend, pBeamFalloff,
						 pHalo, pRes, pBlur, pFogDensity, pSamples, pFog])

	def setPhotonLayout(self):
		self.clearSizer()
		# From
		pFrom, pFromXYZ	= self.locationPanels('From', ['self.txtNameFX', 'self.txtNameFY', 'self.txtNameFZ'],
											  ['self.txtNameFXV', 'self.txtNameFYV', 'self.txtNameFZV'])
		# To
		pTo, pToXYZ	= self.locationPanels('To', ['self.txtNameTX', 'self.txtNameTY', 'self.txtNameTZ'],
										  ['self.txtNameTXV', 'self.txtNameTYV', 'self.txtNameTZV'])
		# Photons
		pPhotons	= wxPanel(self, -1)
		self.txtPhotons = wxTextCtrl(pPhotons, -1, self.txtPhotonsV, validator = NumValidator())
		self.layItOut(pPhotons, [wxStaticText(pPhotons, -1, 'Photons: '), self.txtPhotons])

		# Depth
		pDepth	= wxPanel(self, -1)
		self.txtDepth = wxTextCtrl(pDepth, -1, self.txtDepthV, validator = NumValidator())
		self.layItOut(pDepth, [wxStaticText(pDepth, -1, 'Depth: '), self.txtDepth])

		# Search
		pSearch	= wxPanel(self, -1)
		self.txtSearch = wxTextCtrl(pSearch, -1, self.txtSearchV, validator = NumValidator())
		self.layItOut(pSearch, [wxStaticText(pSearch, -1, 'Search: '), self.txtSearch])

		# Mode
		pMode	= wxPanel(self, -1)
		self.coiMode = wxChoice(pMode, -1, wxDefaultPosition, wxDefaultSize, ['caustic', 'diffuse'])
		if self.coiModeV != '':
			self.coiMode.SetStringSelection(self.coiModeV)
		self.layItOut(pMode, [wxStaticText(pMode, -1, 'Mode: '), self.coiMode])

		# Angle
		pAngle	= wxPanel(self, -1)
		self.txtAngle = wxTextCtrl(pAngle, -1, self.txtAngleV, validator = NumValidator())
		self.layItOut(pAngle, [wxStaticText(pAngle, -1, 'Angle: '), self.txtAngle])

		# Fixedradius
		pFixedradius	= wxPanel(self, -1)
		self.txtFixedradius = wxTextCtrl(pFixedradius, -1, self.txtFixedradiusV, validator = NumValidator())
		self.layItOut(pFixedradius, [wxStaticText(pFixedradius, -1, 'Fixedradius: '), self.txtFixedradius])

		# Cluster
		pCluster	= wxPanel(self, -1)
		self.txtCluster = wxTextCtrl(pCluster, -1, self.txtClusterV, validator = NumValidator())
		self.layItOut(pCluster, [wxStaticText(pCluster, -1, 'Cluster: '), self.txtCluster])

		# use_QMC
		pQMC	= wxPanel(self, -1)
		self.cbQMC = wxCheckBox(pQMC, -1, '')
		self.cbQMC.SetValue(self.cbQMCV)
		self.layItOut(pQMC, [wxStaticText(pQMC, -1, 'Quasi MonteCarlo:'), self.cbQMC])

		self.addToSizer([pFrom, pFromXYZ, pTo, pToXYZ, pPhotons, pDepth, pSearch, pMode, pAngle, pFixedradius, pCluster, pQMC])

	def setAreaLayout(self):
		self.clearSizer()
		for loc in ['A', 'B', 'C', 'D']:
			exec("pLoc"+loc+" = wxPanel(self, -1)")
			exec("self.layItOut(pLoc"+loc+", [wxStaticText(pLoc"+loc+", -1, '"+loc+"')])")
			exec("pLoc"+loc+"XYZ	= wxPanel(self, -1)")
			exec("self.txtName"+loc+"X = wxTextCtrl(pLoc"+loc+"XYZ, -1, self.txtName"+loc+"XV, validator = NumValidator())")
			exec("self.txtName"+loc+"Y = wxTextCtrl(pLoc"+loc+"XYZ, -1, self.txtName"+loc+"YV, validator = NumValidator())")
			exec("self.txtName"+loc+"Z = wxTextCtrl(pLoc"+loc+"XYZ, -1, self.txtName"+loc+"ZV, validator = NumValidator())")
			s1 = "self.layItOut(pLoc"+loc+"XYZ, [wxStaticText(pLoc"+loc+"XYZ, -1, 'X:'), self.txtName"+loc+"X, "
			s2 = "wxStaticText(pLoc"+loc+"XYZ, -1, 'Y:'), self.txtName"+loc+"Y, "
			s3 = "wxStaticText(pLoc"+loc+"XYZ, -1, 'Z:'), self.txtName"+loc+"Z], [1,3,1,3,1,3])"
			cmd = s1+s2+s3
			exec(cmd)

		# Samples
		pSamples	= wxPanel(self, -1)
		self.txtSamples = wxTextCtrl(pSamples, -1, self.txtSamplesV, validator = NumValidator())
		self.layItOut(pSamples, [wxStaticText(pSamples, -1, 'Samples: '), self.txtSamples])

		# PSamples
		pPSamples	= wxPanel(self, -1)
		self.txtPSamples = wxTextCtrl(pPSamples, -1, self.txtPSamplesV, validator = NumValidator())
		self.layItOut(pPSamples, [wxStaticText(pPSamples, -1, 'PSamples: '), self.txtPSamples])

		self.addToSizer([pLocA, pLocAXYZ, pLocB, pLocBXYZ, pLocC, pLocCXYZ, pLocD, pLocDXYZ, pSamples, pPSamples])

	def setHemiLayout(self):
		self.clearSizer()

		# Samples
		pSamples	= wxPanel(self, -1)
		self.txtSamples = wxTextCtrl(pSamples, -1, '', validator = NumValidator())
		self.layItOut(pSamples, [wxStaticText(pSamples, -1, 'Samples: '), self.txtSamples])

		# Randomsampling
		pRandomsampling	= wxPanel(self, -1)
		self.cbRandomsampling = wxCheckBox(pRandomsampling, -1, '')
		self.layItOut(pRandomsampling, [wxStaticText(pRandomsampling, -1, 'Randomsampling:'), self.cbRandomsampling])

		self.addToSizer([pSamples, pRandomsampling])

	def setPathLayout(self):
		self.clearSizer()

		# Samples
		pSamples	= wxPanel(self, -1)
		self.txtSamples = wxTextCtrl(pSamples, -1, '', validator = NumValidator())
		self.layItOut(pSamples, [wxStaticText(pSamples, -1, 'Samples: '), self.txtSamples])

		# Randomsampling
		pRandomsampling	= wxPanel(self, -1)
		self.cbRandomsampling = wxCheckBox(pRandomsampling, -1, '')
		self.layItOut(pRandomsampling, [wxStaticText(pRandomsampling, -1, 'Randomsampling:'), self.cbRandomsampling])

		# Depth
		pDepth	= wxPanel(self, -1)
		self.txtDepth = wxTextCtrl(pDepth, -1, '', validator = NumValidator())
		self.layItOut(pDepth, [wxStaticText(pDepth, -1, 'Depth: '), self.txtDepth])

		# use_QMC
		pQMC	= wxPanel(self, -1)
		self.cbQMC = wxCheckBox(pQMC, -1, '')
		self.cbQMC.SetValue(self.cbQMCV)
		self.layItOut(pQMC, [wxStaticText(pQMC, -1, 'Quasi MonteCarlo:'), self.cbQMC])

		# Cache
		pCache	= wxPanel(self, -1)
		self.cbCache = wxCheckBox(pCache, -1, '')
		self.cbCache.SetValue(self.cbCacheV)
		self.layItOut(pCache, [wxStaticText(pCache, -1, 'Cache:'), self.cbCache])

		# Cache Size
		pCacheSize	= wxPanel(self, -1)
		self.txtCacheSize = wxTextCtrl(pCacheSize, -1, '', validator = NumValidator())
		self.layItOut(pCacheSize, [wxStaticText(pCacheSize, -1, 'Cache Size: '), self.txtCacheSize])

		self.addToSizer([pSamples, pRandomsampling, pDepth, pQMC, pCache, pCacheSize])

	def addToSizer(self, elements):
		'''Does the layout for the dynamic windows of the panel'''
		pos = 6
		for e in elements:
			self.rcs.Add(e, row=pos, col=0, flag=wxEXPAND)
			pos += 1
		self.rcs.Layout()
				
	def clearSizer(self):
		'''Removes the dynamic windows from the panel'''
		rem = self.getRemovables(self.GetChildren())
		for r in rem:
			x = self.rcs.Remove(r)
			r.Destroy()
		
	def getRemovables(self, widlist):
		'''Returns a list of dynamic windows'''
		for x in self.common:
			del widlist[widlist.index(x)]
		return widlist
		
	def getColorList(self, color):
		'''Returns a list with the RGB values (range from 0.0 and 1.0) of the given wxColour'''
		return (color.Red()/255.0, color.Green()/255.0, color.Blue()/255.0)

	def setValues(self, node):
		self.txtName.SetValue(self.master.tree.GetPyData(node).get('attributes', {}).get(u'name', 'default'))
		self.txtPower.SetValue(self.master.tree.GetPyData(node).get('attributes', {}).get(u'power', ''))
		cv = self.getNodeValue(node, 'color')
		if cv: 
			self.bColor.SetBackgroundColour(wxColour(cv[0], cv[1], cv[2]))
		self.bColor.Refresh()
		type = self.master.tree.GetPyData(node).get('attributes', {}).get(u'type', '')
		self.coiType.SetStringSelection(type)
		if self.master.curLight:
			self.stReference.SetLabel('Referencing')
			self.bCopy.Enable(1)
		self.initalizeValues()
		if  type == 'pointlight' or type == 'sunlight':
			self.setPointSunLayout()
		elif  type == 'softlight':
			self.setSoftLayout()
		elif  type == 'spotlight':
			self.setSpotLayout()
		elif  type == 'photonlight':
			self.setPhotonLayout()
		elif  type == 'arealight':
			self.setAreaLayout()
		elif  type == 'hemilight':
			self.setHemiLayout()
		elif  type == 'pathlight':
			self.setPathLayout()

	def	 getNodeValue(self, node, name, type = 'Color', cookie = 0):
		found = 0
		name = string.strip(string.lower(name))
		if self.master.curLight:
			child, nco = self.master.tree.GetFirstChild(node, cookie)
			cc = 0
			while cc < self.master.tree.GetChildrenCount(node, 0) and not found:
				tr = self.master.tree.GetPyData(child)
				if name == tr['name']:
					if type == 'Color':
						rv = [int(float(tr['attributes'][u'r'])*255),
							  int(float(tr['attributes'][u'g'])*255),
							  int(float(tr['attributes'][u'b'])*255)]
					elif type == 'Value':
						rv = float(tr['attributes'][u'value'])
					elif type == 'Coord':
						rv = [float(tr['attributes'][u'x']),
							  float(tr['attributes'][u'y']),
							  float(tr['attributes'][u'z'])]
					found = 1

				child = self.master.tree.GetNextSibling(child)
				cc += 1

			if not found:
				child0, nco = self.master.tree.GetFirstChild(node, cookie)
				if self.master.tree.ItemHasChildren(child0):
					rv = self.getNodeValue(child0, name, type, nco)
				else:
					rv = None
		else:
			rv = None
		return rv

	def onAddToTree(self, evt):
		data = {'name': 'light', 'attributes': {}}
		self.master.curLight = self.master.tree.AppendItem(self.master.tree.treeroot, 'light')
		self.master.tree.SetPyData(self.master.curLight, data)
		self.stReference.SetLabel('Referencing')
		self.bCopy.Enable(1)
		self.copyToTree()
		
	def onCopyToTree(self, evt):
		self.copyToTree()
		
	def copyToTree(self):
		if self.master.curLight:
			tree = self.master.tree
			tree.SelectItem(self.master.curLight)
			oldname = tree.GetPyData(self.master.curLight)['name']
			data = {'name': oldname, 'attributes': {}}
			data['attributes']['name'] = string.strip(self.txtName.GetValue())
			tree.SetPyData(self.master.curLight, data)
			add = ' ' +self.txtName.GetValue()
			tree.SetItemText(self.master.curLight, oldname+add)
			data['attributes']['power'] = string.strip(self.txtPower.GetValue())
			type = self.coiType.GetStringSelection()
			data['attributes']['type'] = self.coiType.GetStringSelection()

			tree.DeleteChildren(self.master.curLight)
			self.appendColor(tree, self.bColor)

			if  type == 'pointlight' or type == 'sunlight':
				if self.cbCShadow.IsChecked():
					data['attributes']['cast_shadows'] = 'on'
				else:
					data['attributes']['cast_shadows'] = 'off'
				self.appendLoc(tree, 'from', 'F')
			elif  type == 'softlight':
				self.appendLoc(tree, 'from', 'F')
				self.getSetAttribute(data, 'res', self.txtRes)
				self.getSetAttribute(data, 'radius', self.txtRadius)
				self.getSetAttribute(data, 'bias', self.txtBias)
			elif  type == 'spotlight':
				self.appendLoc(tree, 'from', 'F')
				self.appendLoc(tree, 'to', 'T')
				self.getSetAttribute(data, 'size', self.txtSize)
				self.getSetAttribute(data, 'blend', self.txtBlend)
				self.getSetAttribute(data, 'beam_falloff', self.txtBeamFalloff)
				if self.cbHalo.IsChecked():
					data['attributes']['halo'] = 'on'
					self.getSetAttribute(data, 'res', self.txtRes)
					self.getSetAttribute(data, 'blur', self.txtBlur)
					self.getSetAttribute(data, 'fog_density', self.txtFogDensity)
					self.getSetAttribute(data, 'samples', self.txtSamples)
					self.appendColor(tree, self.bFog, 'fog')
				else:
					data['attributes']['halo'] = 'off'
			elif  type == 'photonlight':
				self.appendLoc(tree, 'from', 'F')
				self.appendLoc(tree, 'to', 'T')
				data['attributes']['mode'] = string.strip(self.coiMode.GetStringSelection())
				self.getSetAttribute(data , 'photons' , self.txtPhotons)
				self.getSetAttribute(data , 'depth', self.txtDepth)
				self.getSetAttribute(data , 'search', self.txtSearch)
				self.getSetAttribute(data , 'angle', self.txtAngle)
				self.getSetAttribute(data , 'fixedradius', self.txtFixedradius)
				self.getSetAttribute(data , 'cluster', self.txtCluster)
				if self.cbQMC.IsChecked():
					data['attributes']['use_QMC'] = 'on'
				else:
					data['attributes']['use_QMC'] = 'off'
			elif  type == 'arealight':
				for x in ['a','b','c','d']:
					self.appendLoc(tree, x, string.upper(x))
				self.getSetAttribute(data , 'samples', self.txtSamples)
				self.getSetAttribute(data , 'psamples', self.txtPSamples)
			elif  type == 'hemilight':
				self.getSetAttribute(data , 'samples', self.txtSamples)
				if self.cbRandomsampling.IsChecked():
					data['attributes']['randomsampling'] = 'on'
				else:
					data['attributes']['randomsampling'] = 'off'
			elif  type == 'pathlight':
				self.getSetAttribute(data , 'samples', self.txtSamples)
				self.getSetAttribute(data , 'depth', self.txtDepth)
				if self.cbRandomsampling.IsChecked():
					data['attributes']['randomsampling'] = 'on'
				else:
					data['attributes']['randomsampling'] = 'off'
				if self.cbQMC.IsChecked():
					data['attributes']['use_QMC'] = 'on'
				else:
					data['attributes']['use_QMC'] = 'off'
				if self.cbCache.IsChecked():
					data['attributes']['cache'] = 'on'
					self.getSetAttribute(data , 'cache_size', self.txtCacheSize)
				else:
					data['attributes']['cache'] = 'off'

			tree.SetPyData(self.master.curLight, data)
			tree.SortChildren(self.master.tree.GetRootItem())
			self.master.treep.updateViews()
			self.master.notebook.SetSelection(0)
			self.master.setChanged(1)

	def getSetAttribute(self, datalist, attrib, widget):
		val = string.strip(widget.GetValue())
		if val != '':
			datalist['attributes'][attrib] = val
		
	def appendLoc(self, tree, loc, func):
		newchild = tree.AppendItem(self.master.curLight, loc)
		newdata = {'name': loc}
		newattribs = {'x': eval('self.txtName'+func+'X.GetValue()'),
					  'y': eval('self.txtName'+func+'Y.GetValue()'),
					  'z': eval('self.txtName'+func+'Z.GetValue()')}
		newdata['attributes'] = newattribs
		tree.SetPyData(newchild, newdata)

	def appendColor(self, tree, butn, name='color'):
		newchild = tree.AppendItem(self.master.curLight, name)
		newdata = {'name': name}
		cv = self.getColorList(butn.GetBackgroundColour())
		newattribs = {'r': '%s' % cv[0], 'g': '%s' % cv[1], 'b': '%s' % cv[2]} 
		newdata['attributes'] = newattribs
		tree.SetPyData(newchild, newdata)

	def locationPanels(self, loc, txtfields, txtvals):
		pLoc	= wxPanel(self, -1)
		self.layItOut(pLoc, [wxStaticText(pLoc, -1, loc)])
		pLocXYZ	= wxPanel(self, -1)
		exec(txtfields[0]+" = wxTextCtrl(pLocXYZ, -1, "+txtvals[0]+", validator = NumValidator())")
		exec(txtfields[1]+" = wxTextCtrl(pLocXYZ, -1, "+txtvals[1]+", validator = NumValidator())")
		exec(txtfields[2]+" = wxTextCtrl(pLocXYZ, -1, "+txtvals[2]+", validator = NumValidator())")
		s1 = "self.layItOut(pLocXYZ, [wxStaticText(pLocXYZ, -1, 'X:'), "+txtfields[0]+", "
		s2 = "wxStaticText(pLocXYZ, -1, 'Y:'), "+txtfields[1]+", "
		s3 = "wxStaticText(pLocXYZ, -1, 'Z:'), "+txtfields[2]+"], [1,3,1,3,1,3])"
		exec(s1+s2+s3)
		return pLoc, pLocXYZ
