import os, os.path, popen2, string, threading, time, sys, traceback
import PreviewFrame, TreePanel
import Image 
import JpegImagePlugin
import TgaImagePlugin
import PngImagePlugin
from wxPython.wx import *

class Render(threading.Thread):

	def __init__(self, master, output, preview, file, outfile, delete=0, openwin=0):
		threading.Thread.__init__(self)
		Image._initialized = 1
		self.master = master
		self.output = output
		self.preview = preview
		self.file = file
		self.delete = delete
		self.outfile = outfile
		self.openwin = openwin
		self._stopevent = threading.Event()

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def run(self):
		ec1 = None
		ec2 = None
		olddir = os.getcwd()
		if string.strip(os.path.dirname(self.file)) != '':
			os.chdir(os.path.dirname(self.file))
		if self.output != None:
			wxCallAfter(self.output.AppendText, 'Start render [%s]\n' %(time.strftime('%H:%M:%S')))
		starttime=time.time()

		cmd = self.master.config['render.yafex']+' '+int(self.master.config['render.netop'])*'-z '
		if os.name == "mac":
			arg1 = '-c%s' % (self.master.config['render.numcpu'])
		else:
			arg1 = '-c %s' % (self.master.config['render.numcpu'])
		arg2 = '%s' % (os.path.basename(self.file))
		popcmd = cmd + arg1 + " " + arg2

		if os.name != "mac":
			try:
				childouterr, childin = popen2.popen4(popcmd, -1, 'b')
				outerr = childouterr.read(1)
				while outerr and not self._stopevent.isSet():
					if self.output != None:
						for c in outerr:
							if not ord(c) == 13:
								wxMutexGuiEnter()
								self.output.AppendText(c)
								wxMutexGuiLeave()
							else:
								wxMutexGuiEnter()
								lines = self.output.GetNumberOfLines()
								pos = self.output.XYToPosition(0, lines-1)
								self.output.Remove(pos, self.output.GetLastPosition())
								wxMutexGuiLeave()
					outerr = childouterr.read(1)

				ec1 = childouterr.close()
				ec2 = childin.close()
				# sys.stdout.flush()
			except:
				dlg = wxMessageDialog(self,
									  'An error has occured during rendering!'+self.getTraceback(),
									  'ERROR!',
									  wxOK | wxICON_ERROR)
				ok = dlg.ShowModal()

		else:
			if self.output != None:
				wxCallAfter(self.output.AppendText, "Virtual shell cannot be used. Please refer to console.")
			arg = [arg1, arg2]
			print "File", self.file
			print cmd + str(arg)
			os.spawnv(os.P_WAIT, cmd, arg)
				
		if self.output != None:
			wxCallAfter(self.output.AppendText, 'End render	[%s]\n' %(time.strftime('%H:%M:%S')))
			stoptime= time.time()
			elapsed = stoptime - starttime
			ellist=string.split(time.strftime('%j:%H:%M:%S', time.gmtime(elapsed)), ':')
			ellist[0] = '%02i' % (int(ellist[0])-1)
			wxCallAfter(self.output.AppendText, 'Duration:  %.3f sec. [%s]\n' % (elapsed, string.join(ellist,':')))
				
		if not ec1:
			if self.delete:
				try:
					os.remove(os.path.basename(self.file))
				except:
					wxCallAfter(self.showErrorDlg, 'Error: Remove Tempfile!\n')
			try:
				# Show the rendered picture
				wxBell()
				of = os.path.splitext(os.path.basename(self.outfile))[0]+'.jpg'
				rfp = Image.open(self.outfile.encode('ascii')).save(of.encode('ascii'), 'JPEG')
				pw, ph = self.preview.GetClientSize()
				self.master.img = wxImage(of, wxBITMAP_TYPE_JPEG)
				iw = self.master.img.GetWidth()
				ih = self.master.img.GetHeight()
				fact = 1
				if iw > pw or ih > ph:
					if pw < ph:
						fact = (pw*1.0)/iw
					else:
						fact = (ph*1.0)/ih
				wxMutexGuiEnter()
				self.preview.SetBitmap(self.master.img.Scale(fact*iw, fact*ih).ConvertToBitmap())
				wxMutexGuiLeave()
				if isinstance(self.master, TreePanel.TreePanel):
					self.master.validImg = 1
##				if self.openwin:
##					wxCallAfter(self.finished, iw, ih)
			except:
				wxCallAfter(self.showErrorDlg, 'Error: Rendered image!\n')
		else:
			wxCallAfter(self.showErrorDlg, 'Render Error!\n')
			
		os.chdir(olddir)

	def finished(self, iw, ih):
		ps = self.preview.GetSizeTuple()
		if iw > ps[0] or ih > ps[1]:
			win = PreviewFrame.PreviewFrame(self.master, -1, "Preview", self.master.img)
			win.SetSize((iw+20, ih+50))
			win.CenterOnScreen(wxBOTH)
			ok = win.Show(1)

	def showErrorDlg(self, msg):
		type, val, tb = sys.exc_info()
		k = traceback.format_tb(tb, None) + traceback.format_exception_only(type, val)
		traceo = string.join([string.lstrip(l) for l in k])
		dlg = wxMessageDialog(self.master,
							  msg+traceo,
							  'ERROR!',
							  wxOK | wxICON_ERROR)
		ok = dlg.ShowModal()

	def join(self, timeout = None):
		print 'Stoping Thread!'
		self._stopevent.set()
		threading.Thread.join(self, timeout)
