import string
import Render, PreviewFrame, TextOutput
from NumValidator import *
from wxPython.wx import *
from wxPython.lib.buttons import *
from wxPython.lib.evtmgr import eventManager
from wxPython.lib.rcsizer import RowColSizer
from wxPython.lib.colourchooser import wxPyColourChooser

ID_PREVIEW	  = wxNewId()

class ShaderPanel(wxPanel):

	def __init__(self, master, parent, id):
		wxPanel.__init__(self,
						 parent,
						 id,
						 size = parent.GetClientSize(),
						 name='Tree')

		self.master = master
		self.config = self.master.config
		self.render = None
		self.size = 240

		self.chooser = wxPyColourChooser(self, -1)
		self.output = TextOutput.TextOutput(self, -1, size = wxSize(-1, 30), style = wxTE_READONLY|wxTE_MULTILINE)

		# Action buttons
		pAction	  = wxPanel(self, -1)
		self.bCopy = wxButton(pAction, -1, 'Copy To Tree ')
		self.bRender = wxButton(pAction, -1, 'Render')
		self.layItOut(pAction, [self.bCopy, self.bRender])
		self.bCopy.Enable(0)
		
		# Reference
		pReference = wxPanel(self, -1)
		self.stReference = wxStaticText(pReference, -1, 'No Reference', style=wxALIGN_CENTRE|wxST_NO_AUTORESIZE)
		self.bReset = wxButton(pReference, -1, 'Reset')
		self.layItOut(pReference, [self.bReset, self.stReference])

		# Name
		pName	= wxPanel(self, -1)
		self.txtName = wxTextCtrl(pName, -1, 'default')
		self.layItOut(pName, [wxStaticText(pName, -1, 'Name: '), self.txtName])
		
		# Lights
		self.rbxMainLight = wxRadioBox(self, -1, 'Main Light', choices=['SoftLight', 'AreaLight']) 
		self.cbCaust	  = wxCheckBox(self, -1, 'Caustics PhotonLight')
		self.cbDiffuse	  = wxCheckBox(self, -1, 'Diffuse PhotonLight')

		# Antialias
		pAnti	= wxPanel(self, -1)
		self.spAnti = wxSpinCtrl(pAnti, -1, min=1, max=2500, initial=4)
		dum = self.spAnti.GetValue()
		self.spAnti.SetValue(4)
		dum = self.spAnti.GetValue()
		self.layItOut(pAnti, [wxStaticText(pAnti, -1, 'Anti Aliasing: '), self.spAnti])
		
		# Preview
		pPreSize	= wxPanel(self, -1)
		self.spPreSize = wxSpinCtrl(pPreSize, -1, min=32, max=2048, initial=240)
		dum = self.spPreSize.GetValue()
		self.spPreSize.SetValue(240)
		dum = self.spPreSize.GetValue()
		self.layItOut(pPreSize, [wxStaticText(pPreSize, -1, 'Thumb Size: '), self.spPreSize])
		
		self.img = wxImage('default.png', wxBITMAP_TYPE_PNG)
		self.preview = wxStaticBitmap(self, -1,
									  self.img.ConvertToBitmap(),
									  size = (self.size, self.size))
		
		# Colors
		pColor	       = wxPanel(self, -1)
		self.btnColor  = wxGenToggleButton(pColor, -1, 'Color')
		self.btnColor.SetSize(wxSize(self.chooser.GetSize()[0]/6,self.btnColor.GetSize()[1]))
		self.btnSpec   = wxGenToggleButton(pColor, -1, 'Spec ')
		self.btnSpec.SetSize(wxSize(self.chooser.GetSize()[0]/6,self.btnSpec.GetSize()[1]))
		self.cbSpec    = wxCheckBox(pColor, -1, 'On')
		self.btnRef	   = wxGenToggleButton(pColor, -1, ' Ref ')
		self.btnRef.SetSize(wxSize(self.chooser.GetSize()[0]/6,self.btnRef.GetSize()[1]))
		self.cbRef     = wxCheckBox(pColor, -1, 'On')
		self.btnRef2   = wxGenToggleButton(pColor, -1, ' Ref2 ')
		self.btnRef2.SetSize(wxSize(self.chooser.GetSize()[0]/6,self.btnRef2.GetSize()[1]))
		self.btnRef2.SetToolTip(wxToolTip('Reflected color when the incidence angle is near zero'))
		self.cbRef2    = wxCheckBox(pColor, -1, 'On')
		self.btnTrans  = wxGenToggleButton(pColor, -1, 'Trans')
		self.btnTrans.SetSize(wxSize(self.chooser.GetSize()[0]/6,self.btnTrans.GetSize()[1]))
		self.cbTrans   = wxCheckBox(pColor, -1, 'On')
		self.btnTrans2 = wxGenToggleButton(pColor, -1, 'Trans2')
		self.btnTrans2.SetSize(wxSize(self.chooser.GetSize()[0]/6,self.btnTrans2.GetSize()[1]))
		self.btnTrans2.SetToolTip(wxToolTip('Transmitted color when the incidence angle is near zero'))
		
		self.cbTrans2  = wxCheckBox(pColor, -1, 'On')

		self.btnColor.SetValue(1)
		self.current = self.btnColor
		
		pCrcs = RowColSizer()

		pCrcs.Add(self.btnColor, row=0, col=0, flag=wxEXPAND)
		pCrcs.Add(self.btnSpec, row=0, col=1, flag=wxEXPAND)
		pCrcs.Add(self.btnRef, row=0, col=2, flag=wxEXPAND)
		pCrcs.Add(self.btnRef2, row=0, col=3, flag=wxEXPAND)
		pCrcs.Add(self.btnTrans, row=0, col=4, flag=wxEXPAND)
		pCrcs.Add(self.btnTrans2, row=0, col=5, flag=wxEXPAND)
		pCrcs.Add(self.cbSpec, row=1, col=1)
		pCrcs.Add(self.cbRef, row=1, col=2)
		pCrcs.Add(self.cbRef2, row=1, col=3)
		pCrcs.Add(self.cbTrans, row=1, col=4)
		pCrcs.Add(self.cbTrans2, row=1, col=5)
		
		pColor.SetAutoLayout(1)
		pColor.SetSizer(pCrcs)
		pColor.Layout()

		# MeshAttribs
		pMeshAttr	= wxPanel(self, -1)
		self.cbMeshAttr = wxCheckBox(pMeshAttr, -1, 'MeshAttr')
		self.btnTColor = wxGenToggleButton(pMeshAttr, -1, 'TColor')
		self.cbTColor = wxCheckBox(pMeshAttr, -1, 'On')
		self.btnRColor	= wxGenToggleButton(pMeshAttr, -1, 'RColor')
		self.cbRColor = wxCheckBox(pMeshAttr, -1, 'On')

		self.currentMA = self.btnTColor
		
		pMArcs = RowColSizer()

		pMArcs.Add(self.cbMeshAttr, row=0, col=0)
		pMArcs.Add(self.btnTColor, row=0, col=1, flag=wxEXPAND)
		pCrcs.Add(self.cbTColor, row=1, col=1)
		pMArcs.Add(self.btnRColor, row=0, col=2, flag=wxEXPAND)
		pCrcs.Add(self.cbRColor, row=1, col=2)
		
		pMeshAttr.SetAutoLayout(1)
		pMeshAttr.SetSizer(pMArcs)
		pMeshAttr.Layout()

		# Hard, IOR, min_refle
		pHard	= wxPanel(self, -1)
		self.txtHard = wxTextCtrl(pHard, -1, '', validator = NumValidator())
		self.layItOut(pHard, [wxStaticText(pHard, -1, 'Hard: '), self.txtHard])
		pIOR   = wxPanel(self, -1)
		self.txtIOR = wxTextCtrl(pIOR, -1, '', validator = NumValidator())
		self.layItOut(pIOR, [wxStaticText(pIOR, -1, 'IOR: '), self.txtIOR])
		pMinRefl   = wxPanel(self, -1)
		self.txtMinRefl = wxTextCtrl(pMinRefl, -1, '', validator = NumValidator())
		self.layItOut(pMinRefl, [wxStaticText(pMinRefl, -1, 'Min Refle: '), self.txtMinRefl])
		pFFresnel   = wxPanel(self, -1)
		self.cbFFresnel	  = wxCheckBox(pFFresnel, -1, '  ')
		self.layItOut(pFFresnel, [wxStaticText(pFFresnel, -1, 'Fast Fresnel:'), self.cbFFresnel])

		# Sizer
		rcs = RowColSizer()

		rcs.Add(pAction, row=0, col=0, flag=wxEXPAND, colspan = 2)
		rcs.Add(pReference, row=1, col=0, flag=wxEXPAND, colspan = 2)
		rcs.Add(pName, row=2, col=0, flag=wxEXPAND, colspan = 2)
		rcs.Add(self.rbxMainLight, row=3, col=0, colspan = 2)
		rcs.Add(self.cbCaust, row=4, col=0) #, flag=wxEXPAND)
		rcs.Add(self.cbDiffuse, row=5, col=0)
		rcs.Add(pHard, row=6, col=0, flag=wxEXPAND, colspan = 2)
		rcs.Add(pIOR, row=7, col=0, flag=wxEXPAND, colspan = 2)
		rcs.Add(pMinRefl, row=8, col=0, flag=wxEXPAND, colspan = 2)
		rcs.Add(pFFresnel, row=9, col=0, flag=wxEXPAND, colspan = 2)
		rcs.Add(pAnti, row=10, col=0, flag=wxEXPAND, colspan = 2)
		rcs.Add(pPreSize, row=11, col=0, flag=wxEXPAND, colspan = 2)
		rcs.Add(self.preview, row=12, col=0, flag=wxEXPAND, colspan = 3, rowspan = 4, border = 2)
		rcs.Add(self.output, row=16, col=0, flag=wxEXPAND, colspan = 4, rowspan = 3)

		rcs.Add(pColor, row=0, col=3, rowspan = 2, flag=wxEXPAND)
		rcs.Add(pMeshAttr, row=2, col=3, rowspan = 2, flag=wxEXPAND)

		rcs.Add(self.chooser, row=4, col=3, rowspan = 13)

		self.setEventHandling()

		self.SetAutoLayout(1)
		self.SetSizer(rcs)
		self.Layout()

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def setEventHandling(self):
		''' Connects GUI objects with events'''
		eventManager.Register(self.onRender,	  EVT_BUTTON,   self.bRender)
		eventManager.Register(self.onPreviewWin,  EVT_LEFT_UP,  self.preview)
		eventManager.Register(self.onCopyToTree,  EVT_BUTTON,   self.bCopy)
		eventManager.Register(self.onReset,       EVT_BUTTON,   self.bReset)
		eventManager.Register(self.onToggles,	  EVT_BUTTON,   self.btnColor)
		eventManager.Register(self.onToggles,	  EVT_BUTTON,   self.btnSpec)
		eventManager.Register(self.onToggles,	  EVT_BUTTON,   self.btnRef)
		eventManager.Register(self.onToggles,	  EVT_BUTTON,   self.btnRef2)
		eventManager.Register(self.onToggles,	  EVT_BUTTON,   self.btnTrans)
		eventManager.Register(self.onToggles,	  EVT_BUTTON,   self.btnTrans2)
		eventManager.Register(self.onToggles,	  EVT_BUTTON,   self.btnTColor)
		eventManager.Register(self.onToggles,	  EVT_BUTTON,   self.btnRColor)
		eventManager.Register(self.onSpinPreSize, EVT_SPINCTRL,	self.spPreSize)

	def layItOut(self, panel, items, weight=[]):
		sizer = wxBoxSizer(wxHORIZONTAL)
		if not weight:
			weight = len(items)*[1]
		c = 0
		for x in items:
			sizer.Add(x, weight[c], wxEXPAND)
			c += 1
		panel.SetAutoLayout(1)
		panel.SetSizer(sizer)
		panel.Layout()
	
	def getColorList(self, color):
		return (color.Red()/255.0, color.Green()/255.0, color.Blue()/255.0)

	def onPreviewWin(self, evt):
		ps = self.preview.GetSizeTuple()
		cl = [isinstance(x, PreviewFrame.PreviewFrame) for x in self.GetChildren()]
		childs = reduce(lambda x,y: x+y, cl)
		if self.img.GetWidth() > min(ps[0], ps[1]) and not childs:
			win = PreviewFrame.PreviewFrame(self, -1, "Preview", self.img)
			win.SetSize((self.img.GetWidth()+20, self.img.GetWidth()+50))
			win.CenterOnScreen(wxBOTH)
			ok = win.Show(1)
		if childs:
			self.GetChildren()[cl.index(1)].Raise()
			self.GetChildren()[cl.index(1)].SetFocus()

	def onReset(self, evt):
		self.reset()
		
	def reset(self):
		self.stReference.SetLabel('No Reference')
		self.clearValues()
		self.bCopy.Enable(0)
		self.master.curShader = None

	def getSource(self, indent=''):
		shadername = self.txtName.GetValue()
		if string.strip(shadername) == '':
			shadername = "default"
		rv = indent+'<shader type = "generic" name = "%s">\n' % (shadername)
		rv += indent+' <attributes>\n'
		col = self.getColorList(self.btnColor.GetBackgroundColour())
		rv += indent+'  <color r="%s" g="%s" b="%s" />\n' % col
		if self.cbSpec.IsChecked():
			col = self.getColorList(self.btnSpec.GetBackgroundColour())
			rv += indent+'  <specular r="%s" g="%s" b="%s"/>\n' % col
		if self.cbRef.IsChecked():
			col = self.getColorList(self.btnRef.GetBackgroundColour())
			rv += indent+'  <reflected r="%s" g="%s" b="%s"/>\n' % col
		if self.cbRef2.IsChecked():
			col = self.getColorList(self.btnRef2.GetBackgroundColour())
			rv += indent+'  <reflected2 r="%s" g="%s" b="%s"/>\n' % col
		if self.cbTrans.IsChecked():
			col = self.getColorList(self.btnTrans.GetBackgroundColour())
			rv += indent+'  <transmitted r="%s" g="%s" b="%s"/>\n' % col
		if self.cbTrans2.IsChecked():
			col = self.getColorList(self.btnTrans2.GetBackgroundColour())
			rv += indent+'  <transmitted2 r="%s" g="%s" b="%s"/>\n' % col
		if string.strip(self.txtHard.GetValue()) != '':
			rv += indent+'  <hard value ="%s"/>\n' % string.strip(self.txtHard.GetValue())
		if string.strip(self.txtIOR.GetValue()) != '':
			rv += indent+'  <IOR value ="%s"/>\n' % string.strip(self.txtIOR.GetValue())
		if string.strip(self.txtMinRefl.GetValue()) != '':
			rv += indent+'  <min_refle value ="%s"/>\n' % string.strip(self.txtMinRefl.GetValue())
		if self.cbFFresnel.IsChecked():
			rv += indent+'  <fast_fresnel value ="on"/>\n'
		else:
			rv += indent+'  <fast_fresnel value ="off"/>\n'
		
		rv += indent+' </attributes>\n'
		rv += indent+'</shader>\n'
		return rv, shadername

	def onRender(self, evt):
		filename = 'BShaPre.mat'
		outfile = open(filename, 'w')
	
		print >> outfile, '<scene>'
		shadersource, shadername = self.getSource()
		print >> outfile, shadersource
	
		print >> outfile, ' <shader type = "generic" name = "BlackTile">'
		print >> outfile, '	 <attributes>'
		print >> outfile, '	  <color r="0.0" g="0." b="0.0" />'
		print >> outfile, '	  <hard value = "25.0"/>'
		print >> outfile, '	  <min_refle value = "0.2"/>'
		print >> outfile, '	 </attributes>'
		print >> outfile, ' </shader>'

		print >> outfile, ' <shader type = "generic" name = "WhiteTile">'
		print >> outfile, '	 <attributes>'
		print >> outfile, '	  <color r="1.0" g="1.0" b="1.0" />'
		print >> outfile, '	  <hard value = "25.0"/>'
		print >> outfile, '	  <min_refle value = "0.2"/>'
		print >> outfile, '	 </attributes>'
		print >> outfile, ' </shader>'

		#
		# The Meshes: White Tiles
		#
		print >> outfile, ' <transform'
		print >> outfile, '	 m00 = "1.0" m01 = "0.0" m02 = "0.0" m03 = "0.0" '
		print >> outfile, '	 m10 = "0.0" m11 = "1.0" m12 = "0.0" m13 = "0.0" '
		print >> outfile, '	 m20 = "0.0" m21 = "0.0" m22 = "1.0" m23 = "0.0" '
		print >> outfile, '	 m30 = "0.0" m31 = "0.0" m32 = "0.0" m33 = "1.0" >'

		print >> outfile,'	 <object name = "WhiteTile" shader_name = "WhiteTile" >'
		print >> outfile, '	  <attributes>'
		print >> outfile, '	  </attributes>'
		print >> outfile, '	  <mesh>'
		print >> outfile, '	   <points>'

		for i in range(13):
			for j in range(13):
				print >> outfile, '		<p x="'+str(i*2.0-12)+'"  y="'+str(j*2.0-12)+'"	 z="0.0" />'

		print >> outfile, '	   </points>'
		print >> outfile, '	   <faces>'

		for i in range(12):
			for j in range(12):
				if (int((i+j)/2.0)==(i+j)/2.0):
					idx = j + i*13
					print >> outfile, '	   <f a="'+str(idx)+'" b="'+str(idx+1)+'" c="'+str(idx+14)+'" />'
					print >> outfile, '	   <f a="'+str(idx+14)+'" b="'+str(idx+13)+'" c="'+str(idx)+'" />'

		print >> outfile, '	   </faces>'
		print >> outfile, '	  </mesh>'
		print >> outfile, '	 </object>'
		print >> outfile, ' </transform>'

		#
		# The Meshes: Black Tiles
		#
		print >> outfile, ' <transform'
		print >> outfile, '	 m00 = "1.0" m01 = "0.0" m02 = "0.0" m03 = "0.0" '
		print >> outfile, '	 m10 = "0.0" m11 = "1.0" m12 = "0.0" m13 = "-0.0" '
		print >> outfile, '	 m20 = "0.0" m21 = "0.0" m22 = "1.0" m23 = "0.0" '
		print >> outfile, '	 m30 = "0.0" m31 = "0.0" m32 = "0.0" m33 = "1.0" >'

		print >> outfile, '	 <object name = "BlackTile" shader_name = "BlackTile" >'
		print >> outfile, '	  <attributes>'
		print >> outfile, '	  </attributes>'
		print >> outfile, '	  <mesh>'
		print >> outfile, '	   <points>'

		for i in range(13):
			for j in range(13):
				print >> outfile,'	   <p x="'+str(i*2.0-12)+'"	 y="'+str(j*2.0-12)+'"	z="0.0" />'

		print >> outfile,'	  </points>'
		print >> outfile,'	  <faces>'

		for i in range(12):
			for j in range(12):
				if (int((i+j)/2.0)!=(i+j)/2.0):
					idx = j + i*13
					print >> outfile,'	   <f a="'+str(idx)+'" b="'+str(idx+1)+'" c="'+str(idx+14)+'" />'
					print >> outfile,'	   <f a="'+str(idx+14)+'" b="'+str(idx+13)+'" c="'+str(idx)+'" />'

		print >> outfile, '	   </faces>'
		print >> outfile, '	  </mesh>'
		print >> outfile, '	 </object>'
		print >> outfile, ' </transform>'

		#
		# The Sphere
		#
		print >> outfile, ' <transform'
		print >> outfile, '	 m00 = "1.0" m01 = "0.0" m02 = "0.0" m03 = "0.0" '
		print >> outfile, '	 m10 = "0.0" m11 = "1.0" m12 = "0.0" m13 = "0.0" '
		print >> outfile, '	 m20 = "0.0" m21 = "0.0" m22 = "1.0" m23 = "0.0" '
		print >> outfile, '	 m30 = "0.0" m31 = "0.0" m32 = "0.0" m33 = "1.0" >'

		if string.strip(self.txtIOR.GetValue()) != '':
			print >> outfile, '	 <object name="Sphere" shader_name="%s" ' % (shadername),
			print >> outfile, 'caus_IOR="%s" recv_rad = "on" emit_rad ="on">' % string.strip(self.txtIOR.GetValue())
		else:
			print >> outfile,'	<object name="Sphere" shader_name="%s" >' % (shadername)

		print >> outfile, '	  <attributes>'
		
		if self.cbMeshAttr.IsChecked():
			if self.cbTColor.IsChecked():
				col = self.getColorList(self.btnTColor.GetBackgroundColour())
				print >> outfile, '	   <caus_tcolor r="%s" g="%s" b="%s"/>' % col
			if self.cbRColor.IsChecked():
				col = self.getColorList(self.btnRColor.GetBackgroundColour())
				print >> outfile, '	   <caus_rcolor r="%s" g="%s" b="%s"/>' % col
		print >> outfile, '	  </attributes>'
		print >> outfile, '	   <sphere radius="1.0">'
		print >> outfile, '		<center x="0.0" y="-2.0" z="1.0" />'
		print >> outfile, '	   </sphere>'
		print >> outfile, '	 </object>'
		print >> outfile, ' </transform>'

		#
		# Lights
		#
		if self.cbCaust.IsChecked():
			print >> outfile, ' <light type= "photonlight" name="CausticsLight" ',
			print >> outfile, 'power ="1000.0" photons ="200000" depth ="4" search = "100" ',
			print >> outfile, 'angle ="8.0" mode = "caustic" fixedradius = "0.2" cluster ="0.0500000" >'
			print >> outfile, '	 <from x="-6.0" y="-8.0" z="7.0" />'
			print >> outfile, '	 <to x="0.0" y="-2.0" z="2.0" />'
			print >> outfile, '	 <color r="1.0" g="1.0" b="1.0" />'
			print >> outfile, ' </light>'

		if self.cbDiffuse.IsChecked():
			print >> outfile, ' <light type= "photonlight" name="DiffuseLight" ',
			print >> outfile, 'power ="100.0" photons ="2000" depth ="3" search = "200" ',
			print >> outfile, 'angle ="15.0" mode = "diffuse" fixedradius = "1.0" cluster ="0.500000" >'
			print >> outfile, '	 <from x="-6.0" y="-8.0" z="7.0" />'
			print >> outfile, '	 <to x="0.0" y="-2.0" z="2.0" />'
			print >> outfile, '	 <color r="1.0" g="1.0" b="1.0" />'
			print >> outfile, ' </light>'
		
		if self.rbxMainLight.GetSelection() == 1:
			print >> outfile,' <light type ="arealight" name="MainLight" power="15.0" ',
			print >> outfile, 'samples="64.0" psamples="64.0">'
			print >> outfile, '	 <d	 x = "-6.161288"  y = "-7.454182"  z = "7.411288" />'
			print >> outfile, '	 <c	 x = "-6.661288"  y = "-7.954182"  z = "6.704182" />'
			print >> outfile, '	 <b	 x = "-5.954182"  y = "-8.661288"  z = "6.704182" />'
			print >> outfile, '	 <a	 x = "-5.454182"  y = "-8.161288"  z = "7.411288" />'
			print >> outfile, '	 <color r="1.0" g="1.0" b="1.0" />'
			print >> outfile, ' </light>'
		else:
			print >> outfile,' <light type ="softlight" name="MainLight" power="150.0" ',
			print >> outfile, 'res="200" radius="5.0" bias="0.300000" >'
			print >> outfile, '	 <from x="-6.0" y="-8.0" z="7.0" />'
			print >> outfile, '	 <color r="1.0" g="1.0" b="1.0" />'
			print >> outfile, ' </light>'


		print >> outfile, ' <camera name="Camera" resx="%s" resy="%s" focal="1.8" >' % (self.size, self.size) 
		print >> outfile, '	 <from x="0.0" y="-8.128851" z="3.0" />'
		print >> outfile, '	 <to x="-0.0" y="-7.189158" z="2.657980" />'
		print >> outfile, '	 <up x="0.0" y="-7.786831" z="3.939693" />'
		print >> outfile, ' </camera>'

		print >> outfile, ' <render camera_name = "Camera" AA_passes = "%s" ' % self.spAnti.GetValue(),
		print >> outfile, 'AA_threshold = "0.060000" raydepth = "5" bias = "0.300000" ',
		print >> outfile, 'gamma = "1.0" exposure = "0.0"  >'

		name = self.txtName.GetValue()
		if string.strip(name) == '':
			name = "default"
		print >> outfile, '	 <outfile value="%s.tga"/>' % name
		print >> outfile,' </render>'
		print >> outfile, '</scene>'

		outfile.close()

		if name not in self.master.remove:
			self.master.remove.append(name)

		if not self.render or not self.render.isAlive():
			self.render = Render.Render(self,
										self.output,
										self.preview,
										filename,
										name+'.tga', 0, self.size)
			self.render.start()
		else:
			print 'New thread blocked!'
			
	def onSpinPreSize(self, evt):
		self.size = self.spPreSize.GetValue()

	def onToggles(self, evt):
		tb = [self.btnColor, self.btnSpec, self.btnRef, self.btnRef2, self.btnTrans,
			  self.btnTrans2, self.btnTColor, self.btnRColor]
		cb = {self.btnSpec: self.cbSpec, self.btnRef: self.cbRef, self.btnRef2: self.cbRef2,
			  self.btnTrans: self.cbTrans, self.btnTrans2: self.cbTrans2, self.btnTColor: self.cbTColor,
			  self.btnRColor: self.cbRColor}
		mtb = [self.btnTColor, self.btnRColor]
		if evt.GetEventObject().GetValue() == 0:
			evt.GetEventObject().SetValue(1)
			evt.GetEventObject().SetBackgroundColour(self.chooser.GetValue())
			evt.GetEventObject().Refresh()
		else:
			self.chooser.UpdateColour(evt.GetEventObject().GetBackgroundColour())
			del tb[tb.index(evt.GetEventObject())]
			for b in tb:
				b.SetValue(0)
			if evt.GetEventObject() in cb:
				cb[evt.GetEventObject()].SetValue(1)
			if evt.GetEventObject() in mtb:
				self.cbMeshAttr.SetValue(1)

	def setValues(self, node, obj=None):
		name = self.master.tree.GetPyData(node).get('attributes', {}).get(u'name', 'default')
		self.txtName.SetValue(name)
		cv = self.getNodeValue(node, 'color')
		self.btnColor.SetBackgroundColour(wxColour(cv[0], cv[1], cv[2]))
		self.btnColor.Refresh()
		self.chooser.SetValue(wxColour(cv[0], cv[1], cv[2]))
		clist = [['specular', self.btnSpec, self.cbSpec],
				 ['reflected', self.btnRef, self.cbRef],
				 ['reflected2', self.btnRef2, self.cbRef2]]
		for cl in clist:
			cv = self.getNodeValue(node, cl[0])
			if cv:
				cl[1].SetBackgroundColour(wxColour(cv[0], cv[1], cv[2]))
				cl[2].SetValue(1)
			else:
				cl[1].SetBackgroundColour(wxSystemSettings_GetColour(wxSYS_COLOUR_BTNFACE))
				cl[2].SetValue(0)
		cv1 = self.getNodeValue(node, 'transmited')
		cv2 = self.getNodeValue(node, 'transmitted')
		cv = cv1 or cv2
		if cv:
			self.btnTrans.SetBackgroundColour(wxColour(cv[0], cv[1], cv[2]))
			self.cbTrans.SetValue(1)
		else:
			self.btnTrans.SetBackgroundColour(wxSystemSettings_GetColour(wxSYS_COLOUR_BTNFACE))
			self.cbTrans.SetValue(0)
		cv1 = self.getNodeValue(node, 'transmited2')
		cv2 = self.getNodeValue(node, 'transmitted2')
		cv = cv1 or cv2 
		if cv:
			self.btnTrans2.SetBackgroundColour(wxColour(cv[0], cv[1], cv[2]))
			self.cbTrans2.SetValue(1)
		else:
			self.btnTrans2.SetBackgroundColour(wxSystemSettings_GetColour(wxSYS_COLOUR_BTNFACE))
			self.cbTrans2.SetValue(0)

		clist = [['hard', self.txtHard], ['IOR', self.txtIOR], ['min_refle', self.txtMinRefl]]
		for cl in clist:
			val = self.getNodeValue(node, cl[0], 'Value')
			if val:
				cl[1].SetValue('%s' % val)
			else:
				cl[1].SetValue('')

		val = self.getNodeValue(node, 'fast_fresnel', 'Bool')
		if val:
			self.cbFFresnel.SetValue(1)
		else:
			self.cbFFresnel.SetValue(0)
			
		# Mesh Attributes
		if obj:
			tc = self.getNodeValue(obj, 'caus_tcolor')
			rc = self.getNodeValue(obj, 'caus_rcolor')
			if tc or rc:
				self.cbMeshAttr.SetValue(1)
			else:
				self.cbMeshAttr.SetValue(0)
			if tc:
				self.btnTColor.SetBackgroundColour(wxColour(tc[0], tc[1], tc[2]))
				self.cbTColor.SetValue(1)
			else:
				self.btnTColor.SetBackgroundColour(wxSystemSettings_GetColour(wxSYS_COLOUR_BTNFACE))
				self.cbTColor.SetValue(0)
			if rc:
				self.btnRColor.SetBackgroundColour(wxColour(rc[0], rc[1], rc[2]))
				self.cbRColor.SetValue(1)
			else:
				self.btnRColor.SetBackgroundColour(wxSystemSettings_GetColour(wxSYS_COLOUR_BTNFACE))
				self.cbRColor.SetValue(0)

		if self.master.curShader:
			self.stReference.SetLabel('Referencing')
			self.bCopy.Enable(1)

	def	 getNodeValue(self, node, name, type = 'Color', cookie = 0):
		found = 0
		child, nco = self.master.tree.GetFirstChild(node, cookie)
		cc = 0
		while cc < self.master.tree.GetChildrenCount(node, 0) and not found:
			tr = self.master.tree.GetPyData(child)
			if name == tr['name']:
				if type == 'Color':
					rv = [int(float(tr['attributes'][u'r'])*255),
						  int(float(tr['attributes'][u'g'])*255),
						  int(float(tr['attributes'][u'b'])*255)]
				elif type == 'Value':
					rv = float(tr['attributes'][u'value'])
				elif type == 'Bool':
					rv = tr['attributes'][u'value']
					if string.lower(rv) == 'on':
						rv = 1
					else:
						rv=0
				found = 1

			child = self.master.tree.GetNextSibling(child)
			cc += 1

		if not found:
			child0, nco = self.master.tree.GetFirstChild(node, cookie)
			if self.master.tree.ItemHasChildren(child0):
				rv = self.getNodeValue(child0, name, type, nco)
			else:
				rv = None
		return rv

	def clearValues(self):
		self.txtName.SetValue('default')
		self.btnColor.SetValue(1)
		self.btnColor.SetBackgroundColour(wxSystemSettings_GetColour(wxSYS_COLOUR_BTNFACE))
		self.spAnti.SetValue(4)
		self.spPreSize.SetValue(240)
		self.rbxMainLight.SetSelection(0)
		bt = [self.btnSpec, self.btnRef, self.btnRef2, self.btnTrans, self.btnTrans2,
			  self.cbSpec, self.cbRef, self.cbRef2, self.cbTrans, self.cbTrans2,
			  self.cbMeshAttr, self.btnTColor, self.btnRColor, self.cbTColor, self.cbRColor,
			  self.cbCaust, self.cbDiffuse]
		for b in bt:
			b.SetBackgroundColour(wxSystemSettings_GetColour(wxSYS_COLOUR_BTNFACE))
			b.SetValue(0)

		tf = [self.txtHard, self.txtIOR, self.txtMinRefl]
		for t in tf:
			t.SetValue('')
		self.cbFFresnel.SetValue(0)
			
	def onCopyToTree(self, evt):
		if self.master.curShader:
			tree = self.master.tree
			tree.SelectItem(self.master.curShader)
			data = tree.GetPyData(self.master.curShader)
			ov = data['attributes']['name']
			nv = string.strip(self.txtName.GetValue())
			data['attributes']['name'] = nv 
			tree.SetPyData(self.master.curShader, data)
			add = ' ' +nv
			tree.SetItemText(self.master.curShader, tree.GetPyData(self.master.curShader)['name']+add)

			child, cookie = tree.GetFirstChild(self.master.curShader, 0)
			tree.DeleteChildren(child)

			newchild = tree.AppendItem(child, 'color')
			newdata = {'name': 'color'}
			cv = self.getColorList(self.btnColor.GetBackgroundColour())
			newattribs = {'r': '%s' % cv[0], 'g': '%s' % cv[1], 'b': '%s' % cv[2]} 
			newdata['attributes'] = newattribs
			tree.SetPyData(newchild, newdata)

			li = [[self.cbSpec,	  'specular',	  self.btnSpec],
				  [self.cbRef,	  'reflected',	  self.btnRef],
				  [self.cbRef2,	  'reflected2',	  self.btnRef2],
				  [self.cbTrans,  'transmitted',  self.btnTrans],
				  [self.cbTrans2, 'transmitted2', self.btnTrans2]
				  ]

			for l in li:
				if l[0].IsChecked():
					newchild = tree.AppendItem(child, l[1])
					newdata = {'name': l[1]}
					cv = self.getColorList(l[2].GetBackgroundColour())
					newattribs = {'r': '%s' % cv[0], 'g': '%s' % cv[1], 'b': '%s' % cv[2]} 
					newdata['attributes'] = newattribs
					tree.SetPyData(newchild, newdata)

			li = [[self.txtHard, 'hard'], [self.txtIOR, 'IOR'], [self.txtMinRefl, 'min_refle']]

			for l in li:
				if string.strip(l[0].GetValue()) != '':
					  newchild = tree.AppendItem(child, l[1])
					  newdata = {'name': l[1]}
					  newattribs = {'value': string.strip(l[0].GetValue())} 
					  newdata['attributes'] = newattribs
					  tree.SetPyData(newchild, newdata)

			if string.strip(self.txtIOR.GetValue()) != '':
				objdata = tree.GetPyData(self.master.curObj)
				objdata['attributes']['caus_IOR'] = string.strip(self.txtIOR.GetValue())
				objdata['attributes']['recv_rad'] = 'on'
				objdata['attributes']['emit_rad'] = 'on'

			newchild = tree.AppendItem(child, 'fast_fresnel')
			newdata = {'name': 'fast_fresnel'}
			if self.cbFFresnel.IsChecked():
				newattribs = {'value': 'on'} 
			else:
				newattribs = {'value': 'off'} 
			newdata['attributes'] = newattribs
			tree.SetPyData(newchild, newdata)
				
			if self.cbMeshAttr.IsChecked():
				child, nco = tree.GetFirstChild(self.master.curObj, 0)
				# ugly, have to rewrite this if <object> gets more childs
				if not tree.GetPyData(child)['name'] ==	 u'attributes':
					child = tree.GetNextSibling(child) 
				tree.DeleteChildren(child)
				caus = [[self.cbTColor, 'caus_tcolor', self.btnTColor], [self.cbRColor, 'caus_rcolor', self.btnRColor]]
				for c in caus:
					if c[0].IsChecked():
						col = self.getColorList(c[2].GetBackgroundColour())
						newchild = tree.AppendItem(child, c[1])
						newdata = {'name': c[1]}
						cv = self.getColorList(c[2].GetBackgroundColour())
						newattribs = {'r': '%s' % cv[0], 'g': '%s' % cv[1], 'b': '%s' % cv[2]} 
						newdata['attributes'] = newattribs
						tree.SetPyData(newchild, newdata)
					
			tree.SortChildren(self.master.tree.GetRootItem())
			self.master.treep.updateShaders(tree.GetRootItem(), 0, ov, nv)
			self.master.treep.updateViews()
			self.master.notebook.SetSelection(0)
			self.master.setChanged(1)
