import os, os.path, string, sys, time, threading, traceback
import ConfigParser, TextOutput, PreviewFrame
from AttribGrid import *

from xml.sax import make_parser
from xml.sax.handler import ContentHandler

from xml.dom.ext.reader.Sax2 import FromXmlFile

from wxPython.wx import *
from wxPython.lib.evtmgr import eventManager

ID_PREVIEW	  = wxNewId()

SELFCLOSING = ('input', 'depth', 'color1', 'color2', 'filename', 'color',
			   'specular', 'reflection', 'hard', 'transmission', 'IOR',
			   'min_refle', 'reflected', 'transmited', 'transmitted',
			   'from', 'to', 'a', 'b', 'c', 'd', 'p', 'f', 'caus_tcolor',
			   'caus_rcolor', 'center', 'include', 'up', 
			   'outfile', 'fog', 'fast_fresnel', 'normal', 'ringscale_x',
			   'ringscale_y', 'turbulence', 'sharpness', 'background_name',
			   'indirect_samples', 'indirect_power', 'exposure', 'save_alpha',
			   'gamma', 'fog_density', 'fog_color')


class TreePanel(wxPanel):

	def __init__(self, master, parent, id):
		wxPanel.__init__(self,
						 parent,
						 id,
						 size = parent.GetClientSize(),
						 name='Tree')

		self.changed  = 0
		self.master	  = master
		self.config   = self.master.config
		self.validImg = 0
		
		#
		# Split the main window
		splitMAIN	= wxSplitterWindow(self, -1, style = wxSP_LIVE_UPDATE)
		splitMAOP	= wxSplitterWindow(splitMAIN, -1, style = wxSP_3DSASH|wxSP_LIVE_UPDATE)
		splitATCO	= wxSplitterWindow(splitMAOP, -1, style = wxSP_3DSASH|wxSP_LIVE_UPDATE)

		# The DOM Tree
		tID = wxNewId()
		self.tree = DOMTreeCtrl(self, splitMAOP, tID, wxDefaultPosition, wxDefaultSize,
								wxTR_HAS_BUTTONS
								#| wxTR_MULTIPLE
								#| wxTR_MULTIPLE
								#| wxTR_HIDE_ROOT
								)
		
		self.tree.createDOMTree()
		# self.tree.SetPyData(self.treeroot, None)

		if self.tree.treeroot:
			self.tree.Expand(self.tree.treeroot)
		
		# The attribute-list
		tID = wxNewId()
		self.attrgrid = AttribGrid(self, splitATCO, tID,
									   style=wxLC_REPORT)#|wxLC_VRULES|wxLC_HRULES)
		
		# The preview
		self.img = wxImage('default.png', wxBITMAP_TYPE_PNG)
		self.preview = wxStaticBitmap(splitATCO, ID_PREVIEW,
									  self.img.ConvertToBitmap(),
									  wxDefaultPosition, wxDefaultSize, style=wxSUNKEN_BORDER)

		# The output
		self.output = TextOutput.TextOutput(splitMAIN, -1, style = wxTE_READONLY|wxTE_MULTILINE)
		
		splitATCO.SetMinimumPaneSize(20)
		splitATCO.SplitHorizontally(self.attrgrid, self.preview, -160)
		
		splitMAOP.SetMinimumPaneSize(20)
		splitMAOP.SplitVertically(self.tree, splitATCO, -380)
		
		splitMAIN.SetMinimumPaneSize(20)
		splitMAIN.SplitHorizontally(splitMAOP, self.output, -120)
		
		box = wxBoxSizer(wxVERTICAL)
		box.Add(splitMAIN, 1, wxEXPAND)

		eventManager.Register(self.onLeftUp,  EVT_LEFT_UP,	   self.preview)

		self.SetAutoLayout(1)
		self.SetSizer(box)
		self.Layout()

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def __str__(self):
		return "TreePanel!"

	def reset(self):
		self.validImg = 0

	def onLeftUp(self, evt):
		cl = [isinstance(x, PreviewFrame.PreviewFrame) for x in self.GetChildren()]
		childs = reduce(lambda x,y: x+y, cl)
		if self.validImg and not childs:
			win = PreviewFrame.PreviewFrame(self, -1, "Preview", self.img)
			win.SetSize((self.img.GetWidth()+20, self.img.GetHeight()+50))
			win.CenterOnScreen(wxBOTH)
			ok = win.Show(1)
		if childs:
			self.GetChildren()[cl.index(1)].Raise()
			self.GetChildren()[cl.index(1)].SetFocus()
		evt.Skip()


	def updateShaders(self, root, cookie, oldval, newval):
		'''Update object-nodes, if shader name has changed'''
		if self.tree.ItemHasChildren(root):
			child , nco = self.tree.GetFirstChild(root, cookie)
			for n in xrange ( self.tree.GetChildrenCount ( root, 0 ) ):
				data = self.tree.GetPyData(child)
				if data['name'] == u'object':
					sn = data['attributes'].get(u'shader_name', 0)
					if sn == oldval:
						data[u'attributes'][u'shader_name'] =  newval
						self.tree.SetPyData(child, data)
				else:
					self.updateShaders(child, nco, oldval, newval)
				child = self.tree.GetNextSibling(child)

	def updateViews(self):
		''' Updates The Value/Attribute Grid If Another Treenode Has Been Selected'''
		data = self.tree.GetPyData(self.tree.GetSelection())
		if data:
			# self.dataView.SetValue(data.get('text', ''))
			al = data.get('attributes', {})
			keys = []
			for k in self.sortKeys(al.keys()):
				keys.append(k)
			self.attrgrid.ClearGrid()
			self.attrgrid.BeginBatch()

			for x in range(self.attrgrid.GetColSize(0)): # delete old values
				self.attrgrid.SetCellValue(x, 0, '')
				self.attrgrid.SetCellValue(x, 1, '')
				self.attrgrid.SetCellRenderer(0, 1, wxGridCellStringRenderer())
				self.attrgrid.SetCellEditor(x, 1, AttribGridEditor(self))

			if keys == ['r', 'g', 'b']:
				color = (float(al['r']), float(al['g']), float(al['b']))
				colorstring = '(%s, %s, %s)' % color
				self.attrgrid.SetCellValue(0, 0, 'color')
				self.attrgrid.SetCellValue(0, 1, colorstring)
				self.attrgrid.SetCellRenderer(0, 1, AttribGridColorRenderer(color))
				self.attrgrid.SetCellEditor(0, 1, AttribGridColorEditor(self, color))
			else:
				idx = 0
				for x in keys:
					self.attrgrid.SetCellValue(idx, 0, x)
					self.attrgrid.SetCellValue(idx, 1, al[x])
					idx += 1

			self.attrgrid.EndBatch()
			self.attrgrid.AdjustScrollbars()
		else:
			self.attrgrid.Clear()
			#self.dataView.SetValue('')

	def sortKeys(self, keys):
		rv = []
		if 'r' in keys and 'g' in keys and 'b' in keys:
			rv = ['r' ,'g', 'b']
			del keys[keys.index('r')]
			del keys[keys.index('g')]
			del keys[keys.index('b')]

		keys.sort()
		for k in keys:
			rv.append(k)
		return rv
					
	def attribToDict(self, attrib):
		rv = {}
		x = 0
		for x in attrib.getNames():
			rv[x] = attrib.getValue(x)
		return rv
			
class DOMTreeCtrl(wxTreeCtrl):
	
	def __init__(self, master, *args, **kwargs):
		apply(wxTreeCtrl.__init__,(self,) + args, kwargs)
		self.master = master
		self.last = None
		self.changing = 0
		self.treeroot = None
		self.count = 0
		eventManager.Register(self.onTreeSelChanged,  EVT_TREE_SEL_CHANGED,	   self)
		eventManager.Register(self.onTreeSelChanging, EVT_TREE_SEL_CHANGING,   self)
		eventManager.Register(self.onTreeActivate,	  EVT_TREE_ITEM_ACTIVATED, self)
		eventManager.Register(self.onKeyDown,		  EVT_TREE_KEY_DOWN,	   self)
		eventManager.Register(self.onLeftDClick,      EVT_LEFT_DCLICK,	       self)

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def createDOMTree(self):
		if self.master.master.file:

			try:
				self.master.master.GetStatusBar().SetStatusText('Counting Elements')
				ch = CountHandler(self)
				saxparser = make_parser()
				saxparser.setContentHandler(ch)
				saxparser.parse(os.path.join(self.master.master.dirname, self.master.master.file))
				self.master.master.GetStatusBar().SetStatusText('')

				tags = ch.tags.keys()
				tags.sort()
				for tag in tags:
					self.count += ch.tags[tag]

				yh = YafrayHandler(self, self.count)
				saxparser = make_parser()
				saxparser.setContentHandler(yh)
				saxparser.parse(os.path.join(self.master.master.dirname, self.master.master.file))
				self.SortChildren(self.GetRootItem())
			except:
				type, val, tb = sys.exc_info()
				k = traceback.format_tb(tb, None) + traceback.format_exception_only(type, val)
				traceo = string.join([string.lstrip(l) for l in k])
				dlg = wxMessageDialog(self.master,
									  'Error in XML file\n'+traceo,
									  'ERROR!',
									  wxOK | wxICON_ERROR)
				ok = dlg.ShowModal()
				
	def save(self, fn, reset = 1):
		try:
			file = open(fn, 'w')
			root = self.GetRootItem()
			print >> file,'<'+self.GetItemText(root)+'>'
			if self.ItemHasChildren(root):
				self.traverseTree(file, root, 1, '	')
			print >> file,'</'+self.GetItemText(root)+'>'
			if reset:
				self.master.master.setChanged(0)
			file.close()
		except:
			type, val, tb = sys.exc_info()
			k = traceback.format_tb(tb, None) + traceback.format_exception_only(type, val)
			traceo = string.join([string.lstrip(l) for l in k])
			dlg = wxMessageDialog(self,
								  'An error occured writing the file!\n'+traceo,
								  'ERROR!',
								  wxOK | wxICON_ERROR)
			ok = dlg.ShowModal()
			
	def onTreeSelChanged(self, evt):
		self.last = evt.GetOldItem()
		self.changing = 1
		self.master.updateViews()
		self.changing = 0
		
	def onTreeSelChanging(self, evt):
		self.last = evt.GetOldItem()
		
	def onTreeActivate(self, e):
		self.master.updateViews()

	def onLeftDClick(self, evt):
		'''Handles LMB doubleclicks: Either copy to shaderpanel
		or copy to lightpanel, if a shader or a light is doubleclicked.
		Otherwise do nothing '''
		if not self.master.master.copyToShader():
			self.master.master.copyToLight()
		evt.Skip()

	def onKeyDown(self, e):
		# For some reasons EVT_TREE_DELETE_ITEM isn't activated until the end of the application
		if e.GetKeyCode() == WXK_DELETE:
			if self.GetSelection() != self.GetRootItem():
				self.Delete(self.GetSelection())
			else:
				self.DeleteChildren(self.GetSelection())
			self.master.master.setChanged(1)
		else:
			e.Skip()
			
	def onCollapse(self, evt):
		self.collapseNode(self.GetRootItem())

	def onCollaC(self, evt):
		node = self.GetSelection()
		if node.IsOk():
			self.collapseNode(node)

	def onExpand(self, evt):
		self.expandNode(self.GetRootItem())
		
	def onExpandC(self, evt):
		node = self.GetSelection()
		if node.IsOk():
			self.expandNode(node)

	def collapseNode(self, node):
		self.treeWork(self.Collapse, node, 0)
		self.SelectItem(node)
		
	def expandNode(self, node):
		self.treeWork(self.Expand, node, 0)

	def treeWork(self, func, inital, cookie):
		if inital != self.GetRootItem():
			func(inital)
		getter = self.GetFirstChild
		nc = self.GetChildrenCount ( inital, 0 ) 
		for i in xrange ( nc ):
			child, cookie = getter(inital, cookie)
			getter = self.GetNextChild
			func(child)
			if self.GetChildrenCount(child) > 0:
				self.treeWork(func, child, cookie)

	def traverseTree(self, file, root, cookie, spacing):
		if self.ItemHasChildren(root):
			child, nco = self.GetFirstChild(root, cookie)
			for n in xrange(self.GetChildrenCount(root, 0)):
				tr = self.GetPyData(child)
				print >> file,spacing+'<'+tr['name'],
				al = tr.get('attributes', {})
				keys = []
				for k in self.master.sortKeys(al.keys()):
					keys.append(k)
				for x in keys:
					print >> file, x+'="'+al[x]+'"',
				if tr['name'] in SELFCLOSING:
					print >> file,'/>'
				else:
					print >> file,'>'
					self.traverseTree(file, child, nco, spacing+' ')
					print >> file, spacing+'</'+tr['name']+'>'
				child = self.GetNextSibling(child)
	
	def getNodes(self, nodename, root, cookie = 0, erglist=[]):
		if self.ItemHasChildren(root):
			child, nco = self.GetFirstChild(root, cookie)
			for n in xrange(self.GetChildrenCount(root, 0)):
				tr = self.GetPyData(child)
				if nodename != '':
					if tr['name'] == nodename:
						erglist.append(child)
					else:
						erglist = self.getNodes(nodename, child, nco, erglist)
				else:
					erglist.append(child)
				child = self.GetNextSibling(child)
		return erglist

	def OnCompareItems(self, item1, item2):
		compdict = {'shader': 1, 'texture': 0,	  'modulator': 2, 'light': 3,
					'filter': 4, 'transform': 5,  'object': 6,	  'include': 7,
					'camera': 8, 'background': 8, 'render': 10}
		if compdict[self.GetPyData(item1)['name']] ==  compdict[self.GetPyData(item2)['name']]:
			#if self.GetItemText(item1) < self.GetItemText(item2): return -1
			#if self.GetItemText(item1) == self.GetItemText(item2): return 0
			return 0
		if compdict[self.GetPyData(item1)['name']] <  compdict[self.GetPyData(item2)['name']]:
			return -1
		return 1

class YafrayHandler(ContentHandler):

	def __init__(self, treectrl, total):
		self.parents = []
		self.tree = treectrl
		self.counter = 0
		self.total = total
		self.dlg = wxProgressDialog("Parsing Scenefile",
									"Processing Element 00000/%5i" % self.total,
									total,
									self.tree.master.master,
									wxPD_APP_MODAL)
		
	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def startElement(self, name, attrs):
		self.dlg.Update(self.counter, "Processing Element %5i/%5i" % (self.counter+1, self.total))

		if len(self.parents) == 0:
			self.tree.treeroot = self.tree.AddRoot(name)
			data = {'name': name}
			data['attributes'] = self.tree.master.attribToDict(attrs)
			self.tree.SetPyData(self.tree.treeroot, data)
			self.parents.append(self.tree.treeroot)
		else:
			data = {'name': name}
			data['attributes'] = self.tree.master.attribToDict(attrs)
			add = data['attributes'].get('name', '')
			if add != '':
				add = ' ' + add
			ct = self.tree.AppendItem(self.parents[-1], name+add)
			self.tree.SetPyData(ct, data)
			if name == 'render':
				self.tree.master.master.outfile =  data['attributes'].get('outfile', 'salida.tga')
			if name == 'outfile':
				self.tree.master.master.outfile =  data['attributes'].get('value', 'salida.tga')
			self.parents.append(ct)
		
		self.counter += 1
		
##		print '%5i/%5i' % (self.counter, self.total),
##		for p in self.parents:
##			print '->%10s' % self.tree.GetPyData(p)['name'],
##		print 

		if self.counter == self.total:
			self.dlg.Destroy()
			self.tree.master.master.Refresh()
			
	def endElement(self, name):
		del self.parents[-1]

class CountHandler(ContentHandler):

	def __init__(self, treectrl):
		self.tags = {}

	def __repr__(self):
		return '<%s, address: %s>' % (self.__class__.__name__, id(self))
		
	def startElement(self, name, attr):
		self.tags[name] = 1 + self.tags.get(name, 0)
